@@ -34,6 +34,10 @@ void *alloca(size_t);
 # include <sys/types.h>
 #endif
 
+#ifdef HAVE_NETINET_IN_H
+# include <netinet/in.h>
+#endif
+
 #ifdef HAVE_ARPA_INET_H
 # include <arpa/inet.h>
 #endif
@@ -42,10 +46,6 @@ void *alloca(size_t);
 # include <arpa/nameser.h>
 #endif
 
-#ifdef HAVE_NETINET_IN_H
-# include <netinet/in.h>
-#endif
-
 #ifdef HAVE_SYS_SOCKET_H
 # include <sys/socket.h>
 #endif
@@ -54,6 +54,8 @@ void *alloca(size_t);
 # include <netdb.h>
 #endif
 
+#include <errno.h>
+
 #include "Ecore.h"
 #include "ecore_private.h"
 #include "ecore_con_private.h"
@@ -204,7 +206,10 @@ ecore_con_info_get(Ecore_Con_Server *svr,
    int fd[2];
 
    if (pipe(fd) < 0)
-     return 0;
+     {
+        ecore_con_event_server_error(svr, strerror(errno));
+        return 0;
+     }
 
    cbdata = calloc(1, sizeof(CB_Data));
    if (!cbdata)
@@ -222,6 +227,7 @@ ecore_con_info_get(Ecore_Con_Server *svr,
                                                  cbdata,
                                                  NULL, NULL)))
      {
+        ecore_con_event_server_error(svr, "Memory allocation failure");
         free(cbdata);
         close(fd[0]);
         close(fd[1]);
@@ -238,11 +244,10 @@ ecore_con_info_get(Ecore_Con_Server *svr,
         unsigned char *tosend = NULL;
         int tosend_len;
         int canonname_len = 0;
-        int err;
 
-        eina_convert_itoa(svr->port, service);
+        eina_convert_itoa(svr->ecs ? svr->ecs->port : svr->port, service);
         /* CHILD */
-        if (!getaddrinfo(svr->name, service, hints, &result) && result)
+        if (!getaddrinfo(svr->ecs ? svr->ecs->ip : svr->name, service, hints, &result) && result)
           {
              if (result->ai_canonname)
                canonname_len = strlen(result->ai_canonname) + 1;
@@ -275,15 +280,15 @@ ecore_con_info_get(Ecore_Con_Server *svr,
                   memcpy(container->service, sbuf, sizeof(container->service));
                }
 
-             err = write(fd[1], tosend, tosend_len);
+             if (write(fd[1], tosend, tosend_len) < 0) perror("write");
           }
 
         if (result)
           freeaddrinfo(result);
 
-        err = write(fd[1], "", 1);
+        if (write(fd[1], "", 1) < 0) perror("write");
         close(fd[1]);
-#ifdef __USE_ISOC99
+#if defined(__USE_ISOC99) && !defined(__UCLIBC__)
         _Exit(0);
 #else
         _exit(0);
@@ -321,7 +326,7 @@ static void
 _ecore_con_info_readdata(CB_Data *cbdata)
 {
    Ecore_Con_Info container;
-   Ecore_Con_Info *recv;
+   Ecore_Con_Info *recv_info;
    unsigned char *torecv;
    int torecv_len;
 
@@ -342,22 +347,22 @@ _ecore_con_info_readdata(CB_Data *cbdata)
         if ((size > 0) &&
             ((size_t)size == torecv_len - sizeof(Ecore_Con_Info)))
           {
-             recv = (Ecore_Con_Info *)torecv;
+             recv_info = (Ecore_Con_Info *)torecv;
 
-             recv->info.ai_addr =
+             recv_info->info.ai_addr =
                (struct sockaddr *)(torecv + sizeof(Ecore_Con_Info));
              if ((size_t)torecv_len !=
-                 (sizeof(Ecore_Con_Info) + recv->info.ai_addrlen))
-               recv->info.ai_canonname = (char *)
-                 (torecv + sizeof(Ecore_Con_Info) + recv->info.ai_addrlen);
+                 (sizeof(Ecore_Con_Info) + recv_info->info.ai_addrlen))
+               recv_info->info.ai_canonname = (char *)
+                 (torecv + sizeof(Ecore_Con_Info) + recv_info->info.ai_addrlen);
              else
-               recv->info.ai_canonname = NULL;
+               recv_info->info.ai_canonname = NULL;
 
-             recv->info.ai_next = NULL;
+             recv_info->info.ai_next = NULL;
 
              if (cbdata->data)
                {
-                  cbdata->cb_done(cbdata->data, recv);
+                  cbdata->cb_done(cbdata->data, recv_info);
                   ecore_con_server_infos_del(cbdata->data, cbdata);
                }
 
@@ -376,6 +381,7 @@ _ecore_con_info_readdata(CB_Data *cbdata)
      {
         if (cbdata->data)
           {
+             ecore_con_event_server_error(cbdata->data, strerror(errno));
              cbdata->cb_done(cbdata->data, NULL);
              ecore_con_server_infos_del(cbdata->data, cbdata);
           }
