@@ -26,14 +26,14 @@ _ecore_evas_directfb_render(Ecore_Evas *ee)
    Ecore_Evas *ee2;
    int rend = 0;
 
-#ifdef BUILD_ECORE_EVAS_SOFTWARE_BUFFER
    EINA_LIST_FOREACH(ee->sub_ecore_evas, ll, ee2)
      {
         if (ee2->func.fn_pre_render) ee2->func.fn_pre_render(ee2);
-        rend |= _ecore_evas_buffer_render(ee2);
+        if (ee2->engine.func->fn_render)
+          rend |= ee2->engine.func->fn_render(ee2);
         if (ee2->func.fn_post_render) ee2->func.fn_post_render(ee2);
      }
-#endif
+
    if (ee->func.fn_pre_render) ee->func.fn_pre_render(ee);
    updates = evas_render_updates(ee->evas);
    if (updates)
@@ -69,7 +69,7 @@ static Ecore_Evas *
 _ecore_evas_directfb_match(DFBWindowID win)
 {
    Ecore_Evas *ee;
-   
+
    ee = eina_hash_find(ecore_evases_hash, _ecore_evas_directfb_winid_str_get(win));
    return ee;
 }
@@ -286,6 +286,8 @@ _ecore_evas_directfb_move(Ecore_Evas *ee, int x, int y)
 static void
 _ecore_evas_directfb_resize(Ecore_Evas *ee, int w, int h)
 {
+   ee->req.w = w;
+   ee->req.h = h;
    if ((w == ee->w) && (h == ee->h)) return;
    ecore_directfb_window_resize(ee->engine.directfb.window, w, h);
    ee->w = w;
@@ -402,6 +404,8 @@ _ecore_evas_directfb_fullscreen_set(Ecore_Evas *ee, int on)
         resized = 1;
         ee->w = w;
         ee->h = h;
+        ee->req.w = ee->w;
+        ee->req.h = ee->h;
         if ((ee->rotation == 90) || (ee->rotation == 270))
           {
              evas_output_size_set(ee->evas, ee->h, ee->w);
@@ -485,8 +489,16 @@ static Ecore_Evas_Engine_Func _ecore_directfb_engine_func =
      NULL,                              /* ignore events */
      NULL,                              /* alpha */
      NULL, //transparent
-     
-     NULL // render
+
+     NULL,
+     NULL,
+     NULL,
+     NULL,
+     NULL,
+     NULL,
+
+     NULL, // render
+     NULL  // screen_geometry_get
 };
 #endif
 
@@ -524,6 +536,10 @@ ecore_evas_directfb_new(const char *disp_name, int windowed, int x, int y, int w
    ee->y = y;
    ee->w = w;
    ee->h = h;
+   ee->req.x = ee->x;
+   ee->req.y = ee->y;
+   ee->req.w = ee->w;
+   ee->req.h = ee->h;
    ee->prop.layer = 1;
    ee->prop.fullscreen = 0;
 
@@ -544,12 +560,20 @@ ecore_evas_directfb_new(const char *disp_name, int windowed, int x, int y, int w
         if (!evas_engine_info_set(ee->evas, (Evas_Engine_Info *)einfo))
           {
              ERR("evas_engine_info_set() for engine '%s' failed.", ee->driver);
+             ecore_evas_free(ee);
+             return NULL;
           }
      }
-   
+   else
+     {
+        ERR("evas_engine_info_set() init engine '%s' failed.", ee->driver);
+        ecore_evas_free(ee);
+        return NULL;
+     }
+
    ee->engine.func->fn_render = _ecore_evas_directfb_render;
    _ecore_evas_register(ee);
-   
+
    if (!ecore_evases_hash)
      ecore_evases_hash = eina_hash_string_superfast_new(NULL);
    eina_hash_add(ecore_evases_hash, _ecore_evas_directfb_winid_str_get(ee->engine.directfb.window->id), ee);
