@@ -2,6 +2,7 @@
 # include <config.h>
 #endif
 
+#include <stdlib.h>
 #include <sys/types.h>
 #include <dirent.h>
 
@@ -20,10 +21,10 @@ static int _ecore_evas_init_count = 0;
 
 static char *ecore_evas_default_display = "0";
 static Eina_List *ecore_evas_input_devices = NULL;
-static Ecore_Event_Handler *ecore_evas_event_handlers[6] = {NULL, NULL, NULL, NULL, NULL, NULL};
+static Ecore_Event_Handler *ecore_evas_event_handlers[4] = {NULL, NULL, NULL, NULL};
 
 static void
-_ecore_evas_mouse_move_process_fb(Ecore_Evas *ee, int x, int y, unsigned int timestamp)
+_ecore_evas_mouse_move_process_fb(Ecore_Evas *ee, int x, int y)
 {
    int fbw, fbh;
 
@@ -50,14 +51,6 @@ _ecore_evas_mouse_move_process_fb(Ecore_Evas *ee, int x, int y, unsigned int tim
                            y - ee->prop.cursor.hot.x,
                            (fbw - ee->w) + ee->w - x - 1 - ee->prop.cursor.hot.y);
      }
-   if (ee->rotation == 0)
-     evas_event_feed_mouse_move(ee->evas, x, y, timestamp, NULL);
-   else if (ee->rotation == 90)
-     evas_event_feed_mouse_move(ee->evas, (fbh - ee->h) + ee->h - y - 1, x, timestamp, NULL);
-   else if (ee->rotation == 180)
-     evas_event_feed_mouse_move(ee->evas, (fbw - ee->w) + ee->w - x - 1, (fbh - ee->h) + ee->h - y - 1, timestamp, NULL);
-   else if (ee->rotation == 270)
-     evas_event_feed_mouse_move(ee->evas, y, (fbw - ee->w) + ee->w - x - 1, timestamp, NULL);
 }
 
 static Ecore_Evas *fb_ee = NULL;
@@ -87,10 +80,10 @@ _ecore_evas_fb_gain(void *data __UNUSED__)
    Eina_List *ll;
    Ecore_Fb_Input_Device *dev;
 
-   if (fb_ee) 
+   if (fb_ee)
      {
         ee = fb_ee;
-        
+
         ee->visible = 1;
         if ((ee->rotation == 90) || (ee->rotation == 270))
           evas_damage_rectangle_add(ee->evas, 0, 0, ee->h, ee->w);
@@ -102,93 +95,58 @@ _ecore_evas_fb_gain(void *data __UNUSED__)
      ecore_fb_input_device_listen(dev, 1);
 }
 
-static Eina_Bool
-_ecore_evas_event_key_down(void *data __UNUSED__, int type __UNUSED__, void *event)
-{
-   Ecore_Evas *ee;
-   Ecore_Fb_Event_Key_Down *e;
-
-   e = event;
-   ee = _ecore_evas_fb_match();
-   if (!ee) return EINA_TRUE; /* pass on event */
-   evas_event_feed_key_down(ee->evas, e->keyname, e->keysymbol, e->key_compose, NULL, (unsigned int)((unsigned long long)(ecore_time_get() * 1000.0) & 0xffffffff), NULL);
-   return EINA_FALSE; /* dont pass it on */
-}
-
-static Eina_Bool
-_ecore_evas_event_key_up(void *data __UNUSED__, int type __UNUSED__, void *event)
-{
-   Ecore_Evas *ee;
-   Ecore_Fb_Event_Key_Up *e;
-
-   e = event;
-   ee = _ecore_evas_fb_match();
-   if (!ee) return EINA_TRUE; /* pass on event */
-   evas_event_feed_key_up(ee->evas, e->keyname, e->keysymbol, e->key_compose, NULL, (unsigned int)((unsigned long long)(ecore_time_get() * 1000.0) & 0xffffffff), NULL);
-   return EINA_FALSE; /* dont pass it on */
-}
-
 static Eina_Bool
 _ecore_evas_event_mouse_button_down(void *data __UNUSED__, int type __UNUSED__, void *event)
 {
    Ecore_Evas *ee;
-   Ecore_Fb_Event_Mouse_Button_Down *e;
-   Evas_Button_Flags flags = EVAS_BUTTON_NONE;
+   Ecore_Event_Mouse_Button *e;
 
    e = event;
    ee = _ecore_evas_fb_match();
-   if (!ee) return EINA_TRUE; /* pass on event */
-   _ecore_evas_mouse_move_process_fb(ee, e->x, e->y, (unsigned int)((unsigned long long)(ecore_time_get() * 1000.0) & 0xffffffff));
-   if (e->double_click) flags |= EVAS_BUTTON_DOUBLE_CLICK;
-   if (e->triple_click) flags |= EVAS_BUTTON_TRIPLE_CLICK;
-   evas_event_feed_mouse_down(ee->evas, e->button, flags, (unsigned int)((unsigned long long)(ecore_time_get() * 1000.0) & 0xffffffff), NULL);
-   return EINA_FALSE; /* dont pass it on */
+   if (!ee) return ECORE_CALLBACK_PASS_ON;
+   _ecore_evas_mouse_move_process_fb(ee, e->x, e->y);
+   return ECORE_CALLBACK_PASS_ON;
 }
 
 static Eina_Bool
 _ecore_evas_event_mouse_button_up(void *data __UNUSED__, int type __UNUSED__, void *event)
 {
    Ecore_Evas *ee;
-   Ecore_Fb_Event_Mouse_Button_Up *e;
+   Ecore_Event_Mouse_Button *e;
 
    e = event;
    ee = _ecore_evas_fb_match();
-   if (!ee) return EINA_TRUE; /* pass on event */
-   _ecore_evas_mouse_move_process_fb(ee, e->x, e->y, (unsigned int)((unsigned long long)(ecore_time_get() * 1000.0) & 0xffffffff));
-   evas_event_feed_mouse_up(ee->evas, e->button, EVAS_BUTTON_NONE, (unsigned int)((unsigned long long)(ecore_time_get() * 1000.0) & 0xffffffff), NULL);
-   return EINA_FALSE; /* dont pass it on */
+   if (!ee) return ECORE_CALLBACK_PASS_ON;
+   _ecore_evas_mouse_move_process_fb(ee, e->x, e->y);
+   return ECORE_CALLBACK_PASS_ON;
 }
 
 static Eina_Bool
 _ecore_evas_event_mouse_move(void *data __UNUSED__, int type __UNUSED__, void *event)
 {
    Ecore_Evas *ee;
-   Ecore_Fb_Event_Mouse_Move *e;
+   Ecore_Event_Mouse_Move *e;
 
    e = event;
    ee = _ecore_evas_fb_match();
-   if (!ee) return EINA_TRUE; /* pass on event */
-   _ecore_evas_mouse_move_process_fb(ee, e->x, e->y, (unsigned int)((unsigned long long)(ecore_time_get() * 1000.0) & 0xffffffff));
-   return EINA_FALSE; /* dont pass it on */
+   if (!ee) return ECORE_CALLBACK_PASS_ON;
+   _ecore_evas_mouse_move_process_fb(ee, e->x, e->y);
+   return ECORE_CALLBACK_PASS_ON;
 }
 
 static Eina_Bool
 _ecore_evas_event_mouse_wheel(void *data __UNUSED__, int type __UNUSED__, void *event)
 {
    Ecore_Evas *ee;
-   Ecore_Fb_Event_Mouse_Wheel *e;
-   unsigned long long event_time;
+   Ecore_Event_Mouse_Wheel *e;
 
    e = event;
    ee = _ecore_evas_fb_match();
-   if (!ee) return EINA_TRUE; /* pass on event */
-   event_time = (unsigned int)((unsigned long long)(ecore_time_get() * 1000.0) & 0xffffffff);
-   _ecore_evas_mouse_move_process_fb(ee, e->x, e->y, event_time);
-   evas_event_feed_mouse_wheel(ee->evas, e->direction, e->wheel, event_time, NULL);
-   return EINA_FALSE; /* dont pass it on */
+   if (!ee) return ECORE_CALLBACK_PASS_ON;
+   _ecore_evas_mouse_move_process_fb(ee, e->x, e->y);
+   return ECORE_CALLBACK_PASS_ON;
 }
 
-/* XXX: unused
 static int
 _ecore_evas_fb_render(Ecore_Evas *ee)
 {
@@ -197,20 +155,19 @@ _ecore_evas_fb_render(Ecore_Evas *ee)
    if (ee->visible)
      {
         Eina_List *updates;
-
-#ifdef BUILD_ECORE_EVAS_SOFTWARE_BUFFER
         Eina_List *ll;
         Ecore_Evas *ee2;
-#endif
+
         if (ee->func.fn_pre_render) ee->func.fn_pre_render(ee);
-#ifdef BUILD_ECORE_EVAS_SOFTWARE_BUFFER
+
         EINA_LIST_FOREACH(ee->sub_ecore_evas, ll, ee2)
           {
              if (ee2->func.fn_pre_render) ee2->func.fn_pre_render(ee2);
-             rend |= _ecore_evas_buffer_render(ee2);
+             if (ee2->engine.func->fn_render)
+               rend |= ee2->engine.func->fn_render(ee2);
              if (ee2->func.fn_post_render) ee2->func.fn_post_render(ee2);
           }
-#endif
+
         updates = evas_render_updates(ee->evas);
         if (updates)
           {
@@ -224,36 +181,32 @@ _ecore_evas_fb_render(Ecore_Evas *ee)
      evas_norender(ee->evas);
    return rend;
 }
-*/
 
 static int
-_ecore_evas_fb_init(int w, int h)
+_ecore_evas_fb_init(Ecore_Evas *ee, int w, int h)
 {
+   Eina_File_Direct_Info *info;
+   Eina_Iterator *ls;
    Ecore_Fb_Input_Device *device;
    Ecore_Fb_Input_Device_Cap caps;
    int mouse_handled = 0;
-   int keyboard_handled = 0;
-
-   DIR *input_dir;
-   struct dirent *input_entry;
 
    _ecore_evas_init_count++;
    if (_ecore_evas_init_count > 1) return _ecore_evas_init_count;
 
+   ecore_event_evas_init();
+
    /* register all input devices */
-   input_dir = opendir("/dev/input/");
-   if (!input_dir) return _ecore_evas_init_count;
+   ls = eina_file_direct_ls("/dev/input/");
 
-   while ((input_entry = readdir(input_dir)))
+   EINA_ITERATOR_FOREACH(ls, info)
      {
-        char device_path[256];
-
-        if (strncmp(input_entry->d_name, "event", 5) != 0)
+        if (strncmp(info->path + info->name_start, "event", 5) != 0)
           continue;
 
-        snprintf(device_path, 256, "/dev/input/%s", input_entry->d_name);
-        if (!(device = ecore_fb_input_device_open(device_path)))
+        if (!(device = ecore_fb_input_device_open(info->path)))
           continue;
+        ecore_fb_input_device_window_set(device, ee);
 
         caps = ecore_fb_input_device_cap_get(device);
 
@@ -269,10 +222,10 @@ _ecore_evas_fb_init(int w, int h)
              ecore_evas_input_devices = eina_list_append(ecore_evas_input_devices, device);
              if (!mouse_handled)
                {
-                  ecore_evas_event_handlers[2]  = ecore_event_handler_add(ECORE_FB_EVENT_MOUSE_BUTTON_DOWN, _ecore_evas_event_mouse_button_down, NULL);
-                  ecore_evas_event_handlers[3]  = ecore_event_handler_add(ECORE_FB_EVENT_MOUSE_BUTTON_UP, _ecore_evas_event_mouse_button_up, NULL);
-                  ecore_evas_event_handlers[4]  = ecore_event_handler_add(ECORE_FB_EVENT_MOUSE_MOVE, _ecore_evas_event_mouse_move, NULL);
-                  ecore_evas_event_handlers[5]  = ecore_event_handler_add(ECORE_FB_EVENT_MOUSE_WHEEL, _ecore_evas_event_mouse_wheel, NULL);
+                  ecore_evas_event_handlers[0]  = ecore_event_handler_add(ECORE_EVENT_MOUSE_BUTTON_DOWN, _ecore_evas_event_mouse_button_down, NULL);
+                  ecore_evas_event_handlers[1]  = ecore_event_handler_add(ECORE_EVENT_MOUSE_BUTTON_UP, _ecore_evas_event_mouse_button_up, NULL);
+                  ecore_evas_event_handlers[2]  = ecore_event_handler_add(ECORE_EVENT_MOUSE_MOVE, _ecore_evas_event_mouse_move, NULL);
+                  ecore_evas_event_handlers[3]  = ecore_event_handler_add(ECORE_EVENT_MOUSE_WHEEL, _ecore_evas_event_mouse_wheel, NULL);
                   mouse_handled = 1;
                }
           }
@@ -281,23 +234,18 @@ _ecore_evas_fb_init(int w, int h)
           {
              ecore_fb_input_device_listen(device,1);
              ecore_evas_input_devices = eina_list_append(ecore_evas_input_devices, device);
-             if (!keyboard_handled)
-               {
-                  ecore_evas_event_handlers[0]  = ecore_event_handler_add(ECORE_FB_EVENT_KEY_DOWN, _ecore_evas_event_key_down, NULL);
-                  ecore_evas_event_handlers[1]  = ecore_event_handler_add(ECORE_FB_EVENT_KEY_UP, _ecore_evas_event_key_up, NULL);
-                  keyboard_handled = 1;
-               }
           }
      }
-   closedir(input_dir);
-   
+   eina_iterator_free(ls);
+
    if (!mouse_handled)
      {
         if (ecore_fb_ts_init())
           {
-             ecore_evas_event_handlers[2]  = ecore_event_handler_add(ECORE_FB_EVENT_MOUSE_BUTTON_DOWN, _ecore_evas_event_mouse_button_down, NULL);
-             ecore_evas_event_handlers[3]  = ecore_event_handler_add(ECORE_FB_EVENT_MOUSE_BUTTON_UP, _ecore_evas_event_mouse_button_up, NULL);
-             ecore_evas_event_handlers[4]  = ecore_event_handler_add(ECORE_FB_EVENT_MOUSE_MOVE, _ecore_evas_event_mouse_move, NULL);
+             ecore_fb_ts_event_window_set(ee);
+             ecore_evas_event_handlers[0]  = ecore_event_handler_add(ECORE_EVENT_MOUSE_BUTTON_DOWN, _ecore_evas_event_mouse_button_down, NULL);
+             ecore_evas_event_handlers[1]  = ecore_event_handler_add(ECORE_EVENT_MOUSE_BUTTON_UP, _ecore_evas_event_mouse_button_up, NULL);
+             ecore_evas_event_handlers[2]  = ecore_event_handler_add(ECORE_EVENT_MOUSE_MOVE, _ecore_evas_event_mouse_move, NULL);
              mouse_handled = 1;
           }
      }
@@ -307,6 +255,7 @@ _ecore_evas_fb_init(int w, int h)
 static void
 _ecore_evas_fb_free(Ecore_Evas *ee)
 {
+   ecore_evas_input_event_unregister(ee);
    if (fb_ee == ee) fb_ee = NULL;
    _ecore_evas_fb_shutdown();
    ecore_fb_shutdown();
@@ -315,6 +264,8 @@ _ecore_evas_fb_free(Ecore_Evas *ee)
 static void
 _ecore_evas_resize(Ecore_Evas *ee, int w, int h)
 {
+   ee->req.w = w;
+   ee->req.h = h;
    if ((w == ee->w) && (h == ee->h)) return;
    ee->w = w;
    ee->h = h;
@@ -336,6 +287,8 @@ _ecore_evas_resize(Ecore_Evas *ee, int w, int h)
 static void
 _ecore_evas_move_resize(Ecore_Evas *ee, int x __UNUSED__, int y __UNUSED__, int w, int h)
 {
+   ee->req.w = w;
+   ee->req.h = h;
    if ((w == ee->w) && (h == ee->h)) return;
    ee->w = w;
    ee->h = h;
@@ -380,6 +333,8 @@ _ecore_evas_rotation_set(Ecore_Evas *ee, int rotation, int resize __UNUSED__)
              tmp = ee->w;
              ee->w = ee->h;
              ee->h = tmp;
+             ee->req.w = ee->w;
+             ee->req.h = ee->h;
           }
         else
           {
@@ -409,10 +364,30 @@ _ecore_evas_rotation_set(Ecore_Evas *ee, int rotation, int resize __UNUSED__)
      evas_damage_rectangle_add(ee->evas, 0, 0, ee->h, ee->w);
    else
      evas_damage_rectangle_add(ee->evas, 0, 0, ee->w, ee->h);
-   _ecore_evas_mouse_move_process_fb(ee, ee->mouse.x, ee->mouse.y, (unsigned int)((unsigned long long)(ecore_time_get() * 1000.0) & 0xffffffff));
+   _ecore_evas_mouse_move_process_fb(ee, ee->mouse.x, ee->mouse.y);
    if (ee->func.fn_resize) ee->func.fn_resize(ee);
 }
 
+static void
+_ecore_evas_show(Ecore_Evas *ee)
+{
+   if (ee->prop.focused) return;
+   ee->prop.focused = 1;
+   evas_focus_in(ee->evas);
+   if (ee->func.fn_focus_in) ee->func.fn_focus_in(ee);
+}
+
+static void
+_ecore_evas_hide(Ecore_Evas *ee)
+{
+   if (ee->prop.focused)
+     {
+        ee->prop.focused = 0;
+        evas_focus_out(ee->evas);
+        if (ee->func.fn_focus_out) ee->func.fn_focus_out(ee);
+     }
+}
+
 static void
 _ecore_evas_object_cursor_del(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
@@ -481,6 +456,8 @@ _ecore_evas_fullscreen_set(Ecore_Evas *ee, int on)
         if ((w != ee->w) || (h != ee->h)) resized = 1;
         ee->w = w;
         ee->h = h;
+        ee->req.w = ee->w;
+        ee->req.h = ee->h;
         evas_output_size_set(ee->evas, ee->w, ee->h);
         evas_output_viewport_set(ee->evas, 0, 0, ee->w, ee->h);
         evas_damage_rectangle_add(ee->evas, 0, 0, ee->w, ee->h);
@@ -490,6 +467,8 @@ _ecore_evas_fullscreen_set(Ecore_Evas *ee, int on)
         if ((ee->engine.fb.real_w != ee->w) || (ee->engine.fb.real_h != ee->h)) resized = 1;
         ee->w = ee->engine.fb.real_w;
         ee->h = ee->engine.fb.real_h;
+        ee->req.w = ee->w;
+        ee->req.h = ee->h;
         evas_output_size_set(ee->evas, ee->w, ee->h);
         evas_output_viewport_set(ee->evas, 0, 0, ee->w, ee->h);
         evas_damage_rectangle_add(ee->evas, 0, 0, ee->w, ee->h);
@@ -512,12 +491,13 @@ _ecore_evas_fb_shutdown(void)
      {
         int i;
 
-        for (i = 0; i < 6; i++)
+        for (i = 0; i < 4; i++)
           {
              if (ecore_evas_event_handlers[i])
                ecore_event_handler_del(ecore_evas_event_handlers[i]);
           }
         ecore_fb_ts_shutdown();
+        ecore_event_evas_shutdown();
      }
    if (_ecore_evas_init_count < 0) _ecore_evas_init_count = 0;
    return _ecore_evas_init_count;
@@ -546,8 +526,8 @@ static Ecore_Evas_Engine_Func _ecore_fb_engine_func =
      _ecore_evas_move_resize,
      _ecore_evas_rotation_set,
      NULL,
-     NULL,
-     NULL,
+     _ecore_evas_show,
+     _ecore_evas_hide,
      NULL,
      NULL,
      NULL,
@@ -571,15 +551,26 @@ static Ecore_Evas_Engine_Func _ecore_fb_engine_func =
      NULL,
      NULL,
      NULL, //transparent
-     
-     NULL // render
+
+     NULL,
+     NULL,
+     NULL,
+     NULL,
+     NULL,
+     NULL,
+
+     NULL, // render
+     NULL  // screen_geometry_get
 };
 #endif
 
 /**
- * To be documented.
- *
- * FIXME: To be fixed.
+ * @brief Create Ecore_Evas using fb backend.
+ * @param disp_name The name of the display to be used.
+ * @param rotation The rotation to be used.
+ * @param w The width of the Ecore_Evas to be created.
+ * @param h The height of the Ecore_Evas to be created.
+ * @return The new Ecore_Evas.
  */
 #ifdef BUILD_ECORE_EVAS_FB
 EAPI Ecore_Evas *
@@ -604,7 +595,7 @@ ecore_evas_fb_new(const char *disp_name, int rotation, int w, int h)
 
    ECORE_MAGIC_SET(ee, ECORE_MAGIC_EVAS);
 
-   _ecore_evas_fb_init(w, h);
+   _ecore_evas_fb_init(ee, w, h);
 
    ee->engine.func = (Ecore_Evas_Engine_Func *)&_ecore_fb_engine_func;
 
@@ -617,11 +608,13 @@ ecore_evas_fb_new(const char *disp_name, int rotation, int w, int h)
    ee->visible = 1;
    ee->w = w;
    ee->h = h;
+   ee->req.w = ee->w;
+   ee->req.h = ee->h;
 
    ee->prop.max.w = 0;
    ee->prop.max.h = 0;
    ee->prop.layer = 0;
-   ee->prop.focused = 1;
+   ee->prop.focused = 0;
    ee->prop.borderless = 1;
    ee->prop.override = 1;
    ee->prop.maximized = 1;
@@ -655,25 +648,23 @@ ecore_evas_fb_new(const char *disp_name, int rotation, int w, int h)
         if (!evas_engine_info_set(ee->evas, (Evas_Engine_Info *)einfo))
           {
              ERR("evas_engine_info_set() for engine '%s' failed.", ee->driver);
+             ecore_evas_free(ee);
+             return NULL;
           }
      }
-   evas_key_modifier_add(ee->evas, "Shift");
-   evas_key_modifier_add(ee->evas, "Control");
-   evas_key_modifier_add(ee->evas, "Alt");
-   evas_key_modifier_add(ee->evas, "Meta");
-   evas_key_modifier_add(ee->evas, "Hyper");
-   evas_key_modifier_add(ee->evas, "Super");
-   evas_key_lock_add(ee->evas, "Caps_Lock");
-   evas_key_lock_add(ee->evas, "Num_Lock");
-   evas_key_lock_add(ee->evas, "Scroll_Lock");
+   else
+     {
+        ERR("evas_engine_info_set() init engine '%s' failed.", ee->driver);
+        ecore_evas_free(ee);
+        return NULL;
+     }
 
-   ee->engine.func->fn_render = _ecore_evas_buffer_render;
+   ecore_evas_input_event_register(ee);
+
+   ee->engine.func->fn_render = _ecore_evas_fb_render;
    _ecore_evas_register(ee);
    fb_ee = ee;
-   
    evas_event_feed_mouse_in(ee->evas, (unsigned int)((unsigned long long)(ecore_time_get() * 1000.0) & 0xffffffff), NULL);
-   evas_focus_in(ee->evas);
-
    return ee;
 }
 #else
