@@ -1,63 +1,42 @@
 #ifndef _ECORE_EVAS_PRIVATE_H
 #define _ECORE_EVAS_PRIVATE_H
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
-
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-
-#ifdef HAVE_SYS_MMAN_H
-# include <sys/mman.h>
-#endif
-
 #include <Evas.h>
+#include <Ecore.h>
 #include <ecore_private.h>
 #include <Ecore_Input.h>
 #include <Ecore_Input_Evas.h>
 
 #define ECORE_MAGIC_EVAS 0x76543211
 
-#ifndef BUILD_ECORE_DIRECTFB
-# undef BUILD_ECORE_EVAS_DIRECTFB
-#endif
-
 #ifdef BUILD_ECORE_EVAS_X11
-# include "Ecore_X.h"
+# include <Ecore_X.h>
+# include <Ecore_X_Atoms.h>
 # ifdef HAVE_ECORE_X_XCB
 #  include <xcb/xcb.h>
-#  ifdef BUILD_ECORE_EVAS_SOFTWARE_XCB
-#   include <Evas_Engine_Software_X11.h>
-#  endif
-#  ifdef BUILD_ECORE_EVAS_XRENDER_XCB
-#   include <xcb/render.h>
-#   include <Evas_Engine_XRender_X11.h>
-#  endif
-#  ifdef BUILD_ECORE_EVAS_SOFTWARE_8_X11
-#   include <Evas_Engine_Software_8_X11.h>
-#  endif
 # endif
 # ifdef HAVE_ECORE_X_XLIB
 #  include <X11/Xlib.h>
 #  include <X11/Xutil.h>
-#  ifdef BUILD_ECORE_EVAS_SOFTWARE_XLIB
-#   include <Evas_Engine_Software_X11.h>
-#  endif
-#  ifdef BUILD_ECORE_EVAS_XRENDER_X11
-#   include <X11/extensions/Xrender.h>
-#   include <Evas_Engine_XRender_X11.h>
-#  endif
-#  ifdef BUILD_ECORE_EVAS_OPENGL_X11
-#    include <Evas_Engine_GL_X11.h>
-#  endif
-#  ifdef BUILD_ECORE_EVAS_SOFTWARE_16_X11
-#   include <Evas_Engine_Software_16_X11.h>
-#  endif
 # endif
 #endif
 
+#ifdef BUILD_ECORE_EVAS_SOFTWARE_X11
+# include <Evas_Engine_Software_X11.h>
+#endif
+
+#ifdef BUILD_ECORE_EVAS_OPENGL_X11
+# include <Evas_Engine_GL_X11.h>
+#endif
+
+#ifdef BUILD_ECORE_EVAS_SOFTWARE_8_X11
+# include <Evas_Engine_Software_8_X11.h>
+#endif
+
+#ifdef BUILD_ECORE_EVAS_SOFTWARE_16_X11
+# include <Evas_Engine_Software_16_X11.h>
+#endif
+
 #ifdef BUILD_ECORE_EVAS_FB
 # include <Evas_Engine_FB.h>
 #endif
@@ -67,7 +46,7 @@
 # include "Ecore_DirectFB.h"
 #endif
 
-#ifdef BUILD_ECORE_EVAS_SOFTWARE_BUFFER
+#if defined(BUILD_ECORE_EVAS_SOFTWARE_BUFFER) || defined(BUILD_ECORE_EVAS_EWS)
 # include <Evas_Engine_Buffer.h>
 #endif
 
@@ -95,9 +74,22 @@
 # include <Evas_Engine_Software_16_WinCE.h>
 #endif
 
-/**
-   Log domain macros and variable
- **/
+#ifdef BUILD_ECORE_EVAS_GL_COCOA
+# include "Ecore_Cocoa.h"
+# include <Evas_Engine_Gl_Cocoa.h>
+#endif
+
+#ifdef BUILD_ECORE_EVAS_WAYLAND_SHM
+# include "Ecore_Wayland.h"
+# include <Evas_Engine_Wayland_Shm.h>
+#endif
+
+#ifdef BUILD_ECORE_EVAS_WAYLAND_EGL
+# include "Ecore_Wayland.h"
+# include <Evas_Engine_Wayland_Egl.h>
+#endif
+
+/** Log domain macros and variables **/
 
 extern int _ecore_evas_log_dom;
 
@@ -131,69 +123,80 @@ extern int _ecore_evas_log_dom;
 #define IDLE_FLUSH_TIME 0.5
 #ifndef _ECORE_EVAS_H
 typedef struct _Ecore_Evas Ecore_Evas;
+typedef void   (*Ecore_Evas_Event_Cb) (Ecore_Evas *ee);
 #endif
 
-typedef struct _Ecore_Evas_Engine      Ecore_Evas_Engine;
+typedef struct _Ecore_Evas_Engine Ecore_Evas_Engine;
 typedef struct _Ecore_Evas_Engine_Func Ecore_Evas_Engine_Func;
 
 struct _Ecore_Evas_Engine_Func
 {
-   void        (*fn_free) (Ecore_Evas *ee);
-   void        (*fn_callback_resize_set) (Ecore_Evas *ee, void (*func) (Ecore_Evas *ee));
-   void        (*fn_callback_move_set) (Ecore_Evas *ee, void (*func) (Ecore_Evas *ee));
-   void        (*fn_callback_show_set) (Ecore_Evas *ee, void (*func) (Ecore_Evas *ee));
-   void        (*fn_callback_hide_set) (Ecore_Evas *ee, void (*func) (Ecore_Evas *ee));
-   void        (*fn_callback_delete_request_set) (Ecore_Evas *ee, void (*func) (Ecore_Evas *ee));
-   void        (*fn_callback_destroy_set) (Ecore_Evas *ee, void (*func) (Ecore_Evas *ee));
-   void        (*fn_callback_focus_in_set) (Ecore_Evas *ee, void (*func) (Ecore_Evas *ee));
-   void        (*fn_callback_focus_out_set) (Ecore_Evas *ee, void (*func) (Ecore_Evas *ee));
-   void        (*fn_callback_mouse_in_set) (Ecore_Evas *ee, void (*func) (Ecore_Evas *ee));
-   void        (*fn_callback_mouse_out_set) (Ecore_Evas *ee, void (*func) (Ecore_Evas *ee));
-   void        (*fn_callback_sticky_set) (Ecore_Evas *ee, void (*func) (Ecore_Evas *ee));
-   void        (*fn_callback_unsticky_set) (Ecore_Evas *ee, void (*func) (Ecore_Evas *ee));
-   void        (*fn_callback_pre_render_set) (Ecore_Evas *ee, void (*func) (Ecore_Evas *ee));
-   void        (*fn_callback_post_render_set) (Ecore_Evas *ee, void (*func) (Ecore_Evas *ee));
-   void        (*fn_move) (Ecore_Evas *ee, int x, int y);
-   void        (*fn_managed_move) (Ecore_Evas *ee, int x, int y);
-   void        (*fn_resize) (Ecore_Evas *ee, int w, int h);
-   void        (*fn_move_resize) (Ecore_Evas *ee, int x, int y, int w, int h);
-   void        (*fn_rotation_set) (Ecore_Evas *ee, int rot, int resize);
-   void        (*fn_shaped_set) (Ecore_Evas *ee, int shaped);
-   void        (*fn_show) (Ecore_Evas *ee);
-   void        (*fn_hide) (Ecore_Evas *ee);
-   void        (*fn_raise) (Ecore_Evas *ee);
-   void        (*fn_lower) (Ecore_Evas *ee);
-   void        (*fn_activate) (Ecore_Evas *ee);
-   void        (*fn_title_set) (Ecore_Evas *ee, const char *t);
-   void        (*fn_name_class_set) (Ecore_Evas *ee, const char *n, const char *c);
-   void        (*fn_size_min_set) (Ecore_Evas *ee, int w, int h);
-   void        (*fn_size_max_set) (Ecore_Evas *ee, int w, int h);
-   void        (*fn_size_base_set) (Ecore_Evas *ee, int w, int h);
-   void        (*fn_size_step_set) (Ecore_Evas *ee, int w, int h);
-   void        (*fn_object_cursor_set) (Ecore_Evas *ee, Evas_Object *obj, int layer, int hot_x, int hot_y);
-   void        (*fn_layer_set) (Ecore_Evas *ee, int layer);
-   void        (*fn_focus_set) (Ecore_Evas *ee, int on);
-   void        (*fn_iconified_set) (Ecore_Evas *ee, int on);
-   void        (*fn_borderless_set) (Ecore_Evas *ee, int on);
-   void        (*fn_override_set) (Ecore_Evas *ee, int on);
-   void        (*fn_maximized_set) (Ecore_Evas *ee, int on);
-   void        (*fn_fullscreen_set) (Ecore_Evas *ee, int on);
-   void        (*fn_avoid_damage_set) (Ecore_Evas *ee, int on);
-   void        (*fn_withdrawn_set) (Ecore_Evas *ee, int withdrawn);
-   void        (*fn_sticky_set) (Ecore_Evas *ee, int sticky);
-   void        (*fn_ignore_events_set) (Ecore_Evas *ee, int ignore);
-   void        (*fn_alpha_set) (Ecore_Evas *ee, int alpha);
-   void        (*fn_transparent_set) (Ecore_Evas *ee, int transparent);
+   void (*fn_free) (Ecore_Evas *ee);
+   void (*fn_callback_resize_set) (Ecore_Evas *ee, Ecore_Evas_Event_Cb func);
+   void (*fn_callback_move_set) (Ecore_Evas *ee, Ecore_Evas_Event_Cb func);
+   void (*fn_callback_show_set) (Ecore_Evas *ee, Ecore_Evas_Event_Cb func);
+   void (*fn_callback_hide_set) (Ecore_Evas *ee, Ecore_Evas_Event_Cb func);
+   void (*fn_callback_delete_request_set) (Ecore_Evas *ee, Ecore_Evas_Event_Cb func);
+   void (*fn_callback_destroy_set) (Ecore_Evas *ee, Ecore_Evas_Event_Cb func);
+   void (*fn_callback_focus_in_set) (Ecore_Evas *ee, Ecore_Evas_Event_Cb func);
+   void (*fn_callback_focus_out_set) (Ecore_Evas *ee, Ecore_Evas_Event_Cb func);
+   void (*fn_callback_mouse_in_set) (Ecore_Evas *ee, Ecore_Evas_Event_Cb func);
+   void (*fn_callback_mouse_out_set) (Ecore_Evas *ee, Ecore_Evas_Event_Cb func);
+   void (*fn_callback_sticky_set) (Ecore_Evas *ee, Ecore_Evas_Event_Cb func);
+   void (*fn_callback_unsticky_set) (Ecore_Evas *ee, Ecore_Evas_Event_Cb func);
+   void (*fn_callback_pre_render_set) (Ecore_Evas *ee, Ecore_Evas_Event_Cb func);
+   void (*fn_callback_post_render_set) (Ecore_Evas *ee, Ecore_Evas_Event_Cb func);
+   void (*fn_move) (Ecore_Evas *ee, int x, int y);
+   void (*fn_managed_move) (Ecore_Evas *ee, int x, int y);
+   void (*fn_resize) (Ecore_Evas *ee, int w, int h);
+   void (*fn_move_resize) (Ecore_Evas *ee, int x, int y, int w, int h);
+   void (*fn_rotation_set) (Ecore_Evas *ee, int rot, int resize);
+   void (*fn_shaped_set) (Ecore_Evas *ee, int shaped);
+   void (*fn_show) (Ecore_Evas *ee);
+   void (*fn_hide) (Ecore_Evas *ee);
+   void (*fn_raise) (Ecore_Evas *ee);
+   void (*fn_lower) (Ecore_Evas *ee);
+   void (*fn_activate) (Ecore_Evas *ee);
+   void (*fn_title_set) (Ecore_Evas *ee, const char *t);
+   void (*fn_name_class_set) (Ecore_Evas *ee, const char *n, const char *c);
+   void (*fn_size_min_set) (Ecore_Evas *ee, int w, int h);
+   void (*fn_size_max_set) (Ecore_Evas *ee, int w, int h);
+   void (*fn_size_base_set) (Ecore_Evas *ee, int w, int h);
+   void (*fn_size_step_set) (Ecore_Evas *ee, int w, int h);
+   void (*fn_object_cursor_set) (Ecore_Evas *ee, Evas_Object *obj, int layer, int hot_x, int hot_y);
+   void (*fn_layer_set) (Ecore_Evas *ee, int layer);
+   void (*fn_focus_set) (Ecore_Evas *ee, int on);
+   void (*fn_iconified_set) (Ecore_Evas *ee, int on);
+   void (*fn_borderless_set) (Ecore_Evas *ee, int on);
+   void (*fn_override_set) (Ecore_Evas *ee, int on);
+   void (*fn_maximized_set) (Ecore_Evas *ee, int on);
+   void (*fn_fullscreen_set) (Ecore_Evas *ee, int on);
+   void (*fn_avoid_damage_set) (Ecore_Evas *ee, int on);
+   void (*fn_withdrawn_set) (Ecore_Evas *ee, int withdrawn);
+   void (*fn_sticky_set) (Ecore_Evas *ee, int sticky);
+   void (*fn_ignore_events_set) (Ecore_Evas *ee, int ignore);
+   void (*fn_alpha_set) (Ecore_Evas *ee, int alpha);
+   void (*fn_transparent_set) (Ecore_Evas *ee, int transparent);
 
-   int         (*fn_render) (Ecore_Evas *ee);
+   void (*fn_window_group_set) (Ecore_Evas *ee, const Ecore_Evas *ee_group);
+   void (*fn_aspect_set) (Ecore_Evas *ee, double aspect);
+   void (*fn_urgent_set) (Ecore_Evas *ee, int urgent);
+   void (*fn_modal_set) (Ecore_Evas *ee, int modal);
+   void (*fn_demands_attention_set) (Ecore_Evas *ee, int demand);
+   void (*fn_focus_skip_set) (Ecore_Evas *ee, int skip);
+
+   int (*fn_render) (Ecore_Evas *ee);
+   void (*fn_screen_geometry_get) (const Ecore_Evas *ee, int *x, int *y, int *w, int *h);
 };
 
 struct _Ecore_Evas_Engine
 {
    Ecore_Evas_Engine_Func *func;
 
+/* TODO: UGLY! This should be an union or inheritance! */
 #ifdef BUILD_ECORE_EVAS_X11
-   struct {
+   struct 
+     {
       Ecore_X_Window win_root;
       Eina_List     *win_extra;
       Ecore_X_Pixmap pmap;
@@ -202,6 +205,9 @@ struct _Ecore_Evas_Engine
       Ecore_X_XRegion *damages;
       Ecore_X_Sync_Counter sync_counter;
       Ecore_X_Window leader;
+      Ecore_X_Sync_Counter netwm_sync_counter;
+      int            netwm_sync_val_hi;
+      unsigned int   netwm_sync_val_lo;
       int            sync_val; // bigger! this will screw up at 2 billion frames (414 days of continual rendering @ 60fps)
       int            screen_num;
       int            px, py, pw, ph;
@@ -210,6 +216,8 @@ struct _Ecore_Evas_Engine
       unsigned char  managed : 1;
       unsigned char  sync_began : 1;
       unsigned char  sync_cancel : 1;
+      unsigned char  netwm_sync_set : 1;
+      unsigned char  configure_coming : 1;
       struct {
 	   unsigned char modal : 1;
 	   unsigned char sticky : 1;
@@ -222,6 +230,7 @@ struct _Ecore_Evas_Engine
 	   unsigned char above : 1;
 	   unsigned char below : 1;
       } state;
+      Ecore_X_Window win_shaped_input;
    } x;
 #endif
 #ifdef BUILD_ECORE_EVAS_FB
@@ -261,6 +270,27 @@ struct _Ecore_Evas_Engine
       } state;
    } wince;
 #endif
+#ifdef BUILD_ECORE_EVAS_EWS
+   struct {
+      Evas_Object *image;
+   } ews;
+#endif
+
+#if defined(BUILD_ECORE_EVAS_WAYLAND_SHM) || defined(BUILD_ECORE_EVAS_WAYLAND_EGL)
+   struct 
+     {
+        Ecore_Wl_Window *parent, *win;
+        Evas_Object *frame;
+
+# if defined(BUILD_ECORE_EVAS_WAYLAND_SHM)
+        struct wl_shm_pool *pool;
+        size_t pool_size;
+        void *pool_data;
+        struct wl_buffer *buffer;
+# endif
+
+     } wl;
+#endif
 
    Ecore_Timer *idle_flush_timer;
 };
@@ -280,6 +310,7 @@ struct _Ecore_Evas
    Eina_Bool   should_be_visible : 1;
    Eina_Bool   alpha  : 1;
    Eina_Bool   transparent  : 1;
+   Eina_Bool   in  : 1;
 
    Eina_Hash  *data;
 
@@ -315,6 +346,9 @@ struct _Ecore_Evas
       int             layer;
       Ecore_Window    window;
       unsigned char   avoid_damage;
+      Ecore_Evas     *group_ee;
+      Ecore_Window    group_ee_win;
+      double          aspect;
       char            focused      : 1;
       char            iconified    : 1;
       char            borderless   : 1;
@@ -324,7 +358,13 @@ struct _Ecore_Evas
       char            withdrawn    : 1;
       char            sticky       : 1;
       char            request_pos  : 1;
-   } prop;
+      char            draw_frame   : 1;
+      char            hwsurface    : 1;
+      char            urgent           : 1;
+      char            modal            : 1;
+      char            demand_attention : 1;
+      char            focus_skip       : 1;
+  } prop;
 
    struct {
       void          (*fn_resize) (Ecore_Evas *ee);
@@ -342,17 +382,25 @@ struct _Ecore_Evas
       void          (*fn_pre_render) (Ecore_Evas *ee);
       void          (*fn_post_render) (Ecore_Evas *ee);
       void          (*fn_pre_free) (Ecore_Evas *ee);
+      void          (*fn_state_change) (Ecore_Evas *ee);
    } func;
 
    Ecore_Evas_Engine engine;
    Eina_List *sub_ecore_evas;
 
+   int refcount;
+
    unsigned char ignore_events : 1;
    unsigned char manual_render : 1;
    unsigned char registered : 1;
    unsigned char no_comp_sync  : 1;
+   unsigned char semi_sync  : 1;
+   unsigned char deleted : 1;
 };
 
+void _ecore_evas_ref(Ecore_Evas *ee);
+void _ecore_evas_unref(Ecore_Evas *ee);
+
 #ifdef BUILD_ECORE_EVAS_X11
 int _ecore_evas_x_shutdown(void);
 #endif
@@ -372,6 +420,26 @@ int _ecore_evas_win32_shutdown(void);
 #ifdef BUILD_ECORE_EVAS_SOFTWARE_16_WINCE
 int _ecore_evas_wince_shutdown(void);
 #endif
+#ifdef BUILD_ECORE_EVAS_EWS
+void _ecore_evas_ews_events_init(void);
+int _ecore_evas_ews_shutdown(void);
+#endif
+
+#ifdef BUILD_ECORE_EVAS_WAYLAND_SHM
+void _ecore_evas_wayland_shm_resize(Ecore_Evas *ee, int location);
+void _ecore_evas_wayland_shm_move(Ecore_Evas *ee, int x, int y);
+void _ecore_evas_wayland_shm_drag_start(Ecore_Evas *ee, Ecore_Evas *drag_ee, void *source);
+void _ecore_evas_wayland_shm_pointer_set(Ecore_Evas *ee, int hot_x, int hot_y);
+void _ecore_evas_wayland_shm_type_set(Ecore_Evas *ee, int type);
+#endif
+
+#ifdef BUILD_ECORE_EVAS_WAYLAND_EGL
+void _ecore_evas_wayland_egl_resize(Ecore_Evas *ee, int location);
+void _ecore_evas_wayland_egl_move(Ecore_Evas *ee, int x, int y);
+void _ecore_evas_wayland_egl_drag_start(Ecore_Evas *ee, Ecore_Evas *drag_ee, void *source);
+void _ecore_evas_wayland_egl_pointer_set(Ecore_Evas *ee, int hot_x, int hot_y);
+void _ecore_evas_wayland_egl_type_set(Ecore_Evas *ee, int type);
+#endif
 
 void _ecore_evas_fps_debug_init(void);
 void _ecore_evas_fps_debug_shutdown(void);
@@ -409,4 +477,7 @@ void _ecore_evas_mouse_multi_up_process(Ecore_Evas *ee, int device,
 
 extern Eina_Bool _ecore_evas_app_comp_sync;
 
+void _ecore_evas_extn_init(void);
+void _ecore_evas_extn_shutdown(void);
+
 #endif
