@@ -19,7 +19,7 @@
 
 #ifdef BUILD_ECORE_EVAS_WIN32
 
-#define ECORE_EVAS_EVENT_COUNT 8
+#define ECORE_EVAS_EVENT_COUNT 10
 
 static int _ecore_evas_init_count = 0;
 
@@ -29,6 +29,10 @@ static Eina_Bool _ecore_evas_win32_event_mouse_in(void *data __UNUSED__, int typ
 
 static Eina_Bool _ecore_evas_win32_event_mouse_out(void *data __UNUSED__, int type __UNUSED__, void *event);
 
+static Eina_Bool _ecore_evas_win32_event_window_focus_in(void *data __UNUSED__, int type __UNUSED__, void *event);
+
+static Eina_Bool _ecore_evas_win32_event_window_focus_out(void *data __UNUSED__, int type __UNUSED__, void *event);
+
 static Eina_Bool _ecore_evas_win32_event_window_damage(void *data __UNUSED__, int type __UNUSED__, void *event);
 
 static Eina_Bool _ecore_evas_win32_event_window_destroy(void *data __UNUSED__, int type __UNUSED__, void *event);
@@ -48,17 +52,17 @@ _ecore_evas_win32_render(Ecore_Evas *ee)
 {
    int rend = 0;
    Eina_List *updates = NULL;
-#ifdef BUILD_ECORE_EVAS_SOFTWARE_BUFFER
    Eina_List *ll;
    Ecore_Evas *ee2;
 
    EINA_LIST_FOREACH(ee->sub_ecore_evas, ll, ee2)
      {
         if (ee2->func.fn_pre_render) ee2->func.fn_pre_render(ee2);
-        rend |= _ecore_evas_buffer_render(ee2);
+        if (ee2->engine.func->fn_render)
+          rend |= ee2->engine.func->fn_render(ee2);
         if (ee2->func.fn_post_render) ee2->func.fn_post_render(ee2);
      }
-#endif
+
    if (ee->func.fn_pre_render) ee->func.fn_pre_render(ee);
    if (ee->prop.avoid_damage)
      {
@@ -93,15 +97,17 @@ _ecore_evas_win32_init(void)
    _ecore_evas_init_count++;
    if (_ecore_evas_init_count > 1)
      return _ecore_evas_init_count;
-   
+
    ecore_evas_event_handlers[0]  = ecore_event_handler_add(ECORE_WIN32_EVENT_MOUSE_IN, _ecore_evas_win32_event_mouse_in, NULL);
    ecore_evas_event_handlers[1]  = ecore_event_handler_add(ECORE_WIN32_EVENT_MOUSE_OUT, _ecore_evas_win32_event_mouse_out, NULL);
-   ecore_evas_event_handlers[2]  = ecore_event_handler_add(ECORE_WIN32_EVENT_WINDOW_DAMAGE, _ecore_evas_win32_event_window_damage, NULL);
-   ecore_evas_event_handlers[3]  = ecore_event_handler_add(ECORE_WIN32_EVENT_WINDOW_DESTROY, _ecore_evas_win32_event_window_destroy, NULL);
-   ecore_evas_event_handlers[4]  = ecore_event_handler_add(ECORE_WIN32_EVENT_WINDOW_SHOW, _ecore_evas_win32_event_window_show, NULL);
-   ecore_evas_event_handlers[5]  = ecore_event_handler_add(ECORE_WIN32_EVENT_WINDOW_HIDE, _ecore_evas_win32_event_window_hide, NULL);
-   ecore_evas_event_handlers[6]  = ecore_event_handler_add(ECORE_WIN32_EVENT_WINDOW_CONFIGURE, _ecore_evas_win32_event_window_configure, NULL);
-   ecore_evas_event_handlers[7]  = ecore_event_handler_add(ECORE_WIN32_EVENT_WINDOW_DELETE_REQUEST, _ecore_evas_win32_event_window_delete_request, NULL);
+   ecore_evas_event_handlers[2]  = ecore_event_handler_add(ECORE_WIN32_EVENT_WINDOW_FOCUS_IN, _ecore_evas_win32_event_window_focus_in, NULL);
+   ecore_evas_event_handlers[3]  = ecore_event_handler_add(ECORE_WIN32_EVENT_WINDOW_FOCUS_OUT, _ecore_evas_win32_event_window_focus_out, NULL);
+   ecore_evas_event_handlers[4]  = ecore_event_handler_add(ECORE_WIN32_EVENT_WINDOW_DAMAGE, _ecore_evas_win32_event_window_damage, NULL);
+   ecore_evas_event_handlers[5]  = ecore_event_handler_add(ECORE_WIN32_EVENT_WINDOW_DESTROY, _ecore_evas_win32_event_window_destroy, NULL);
+   ecore_evas_event_handlers[6]  = ecore_event_handler_add(ECORE_WIN32_EVENT_WINDOW_SHOW, _ecore_evas_win32_event_window_show, NULL);
+   ecore_evas_event_handlers[7]  = ecore_event_handler_add(ECORE_WIN32_EVENT_WINDOW_HIDE, _ecore_evas_win32_event_window_hide, NULL);
+   ecore_evas_event_handlers[8]  = ecore_event_handler_add(ECORE_WIN32_EVENT_WINDOW_CONFIGURE, _ecore_evas_win32_event_window_configure, NULL);
+   ecore_evas_event_handlers[9]  = ecore_event_handler_add(ECORE_WIN32_EVENT_WINDOW_DELETE_REQUEST, _ecore_evas_win32_event_window_delete_request, NULL);
 
    ecore_event_evas_init();
    return _ecore_evas_init_count;
@@ -141,9 +147,9 @@ _ecore_evas_win32_event_mouse_in(void *data __UNUSED__, int type __UNUSED__, voi
    if (ee->func.fn_mouse_in) ee->func.fn_mouse_in(ee);
    /* FIXME to do */
 /*    _ecore_evas_x_modifier_locks_update(ee, e->modifiers); */
-   evas_event_feed_mouse_in(ee->evas, e->time, NULL);
+   evas_event_feed_mouse_in(ee->evas, e->timestamp, NULL);
    evas_focus_in(ee->evas);
-   _ecore_evas_mouse_move_process(ee, e->x, e->y, e->time);
+   _ecore_evas_mouse_move_process(ee, e->x, e->y, e->timestamp);
 
    return 1;
 }
@@ -163,15 +169,49 @@ _ecore_evas_win32_event_mouse_out(void *data __UNUSED__, int type __UNUSED__, vo
 
    /* FIXME to do */
 /*    _ecore_evas_x_modifier_locks_update(ee, e->modifiers); */
-   _ecore_evas_mouse_move_process(ee, e->x, e->y, e->time);
+   _ecore_evas_mouse_move_process(ee, e->x, e->y, e->timestamp);
 
-   evas_event_feed_mouse_out(ee->evas, e->time, NULL);
+   evas_event_feed_mouse_out(ee->evas, e->timestamp, NULL);
    if (ee->func.fn_mouse_out) ee->func.fn_mouse_out(ee);
    if (ee->prop.cursor.object) evas_object_hide(ee->prop.cursor.object);
 
    return 1;
 }
 
+static Eina_Bool
+_ecore_evas_win32_event_window_focus_in(void *data __UNUSED__, int type __UNUSED__, void *event)
+{
+   Ecore_Evas *ee;
+   Ecore_Win32_Event_Window_Focus_In *e;
+
+   e = event;
+   ee = ecore_event_window_match((Ecore_Window)e->window);
+   if ((!ee) || (ee->ignore_events)) return ECORE_CALLBACK_PASS_ON;
+   if ((Ecore_Window)e->window != ee->prop.window) return ECORE_CALLBACK_PASS_ON;
+
+   ee->prop.focused = 1;
+   evas_focus_in(ee->evas);
+   if (ee->func.fn_focus_in) ee->func.fn_focus_in(ee);
+   return ECORE_CALLBACK_PASS_ON;
+}
+
+static Eina_Bool
+_ecore_evas_win32_event_window_focus_out(void *data __UNUSED__, int type __UNUSED__, void *event)
+{
+   Ecore_Evas *ee;
+   Ecore_Win32_Event_Window_Focus_Out *e;
+
+   e = event;
+   ee = ecore_event_window_match((Ecore_Window)e->window);
+   if ((!ee) || (ee->ignore_events)) return ECORE_CALLBACK_PASS_ON;
+   if ((Ecore_Window)e->window != ee->prop.window) return ECORE_CALLBACK_PASS_ON;
+
+   evas_focus_out(ee->evas);
+   ee->prop.focused = 0;
+   if (ee->func.fn_focus_out) ee->func.fn_focus_out(ee);
+   return ECORE_CALLBACK_PASS_ON;
+}
+
 static Eina_Bool
 _ecore_evas_win32_event_window_damage(void *data __UNUSED__, int type __UNUSED__, void *event)
 {
@@ -293,17 +333,26 @@ _ecore_evas_win32_event_window_configure(void *data __UNUSED__, int type __UNUSE
    if (!ee) return 1; /* pass on event */
    if ((Ecore_Window)e->window != ee->prop.window) return 1;
 
-   if ((ee->x != e->x) || (ee->y != e->y))
+   if (ee->prop.override)
      {
-        ee->x = e->x;
-        ee->y = e->y;
-        if (ee->func.fn_move) ee->func.fn_move(ee);
+        if ((ee->x != e->x) || (ee->y != e->y))
+          {
+             ee->x = e->x;
+             ee->y = e->y;
+             ee->req.x = ee->x;
+             ee->req.y = ee->y;
+
+             if (ee->func.fn_move) ee->func.fn_move(ee);
+          }
      }
 
    if ((ee->w != e->width) || (ee->h != e->height))
      {
         ee->w = e->width;
         ee->h = e->height;
+        ee->req.w = ee->w;
+        ee->req.h = ee->h;
+
         if ((ee->rotation == 90) || (ee->rotation == 270))
           {
              evas_output_size_set(ee->evas, ee->h, ee->w);
@@ -316,8 +365,11 @@ _ecore_evas_win32_event_window_configure(void *data __UNUSED__, int type __UNUSE
           }
         if (ee->prop.avoid_damage)
           {
+             int pdam;
+
+             pdam = ecore_evas_avoid_damage_get(ee);
              ecore_evas_avoid_damage_set(ee, 0);
-             ecore_evas_avoid_damage_set(ee, 1);
+             ecore_evas_avoid_damage_set(ee, pdam);
           }
 /*         if (ee->shaped) */
 /*           _ecore_evas_win32_region_border_resize(ee); */
@@ -371,7 +423,7 @@ _ecore_evas_win32_free(Ecore_Evas *ee)
 
 static void
 _ecore_evas_win32_callback_delete_request_set(Ecore_Evas *ee,
-                                              void (*func) (Ecore_Evas *ee))
+                                              Ecore_Evas_Event_Cb func)
 {
    ee->func.fn_delete_request = func;
 }
@@ -380,6 +432,8 @@ static void
 _ecore_evas_win32_move(Ecore_Evas *ee, int x, int y)
 {
   INF("ecore evas move (%dx%d)", x, y);
+   ee->req.x = x;
+   ee->req.y = y;
 
    if ((x != ee->x) || (y != ee->y))
      {
@@ -395,6 +449,8 @@ static void
 _ecore_evas_win32_resize(Ecore_Evas *ee, int width, int height)
 {
    INF("ecore evas resize (%dx%d)", width, height);
+   ee->req.w = width;
+   ee->req.h = height;
 
    if ((ee->w != width) || (ee->h != height))
      {
@@ -430,6 +486,10 @@ static void
 _ecore_evas_win32_move_resize(Ecore_Evas *ee, int x, int y, int width, int height)
 {
    INF("ecore evas resize (%dx%d %dx%d)", x, y, width, height);
+   ee->req.x = x;
+   ee->req.y = y;
+   ee->req.w = width;
+   ee->req.h = height;
 
    if ((ee->w != width) || (ee->h != height) || (x != ee->x) || (y != ee->y))
      {
@@ -660,7 +720,7 @@ _ecore_evas_win32_activate(Ecore_Evas *ee)
 {
    INF("ecore evas activate");
 
-   ecore_win32_window_focus_set((struct _Ecore_Win32_Window *)ee->prop.window);
+   ecore_win32_window_focus((struct _Ecore_Win32_Window *)ee->prop.window);
 }
 
 static void
@@ -762,7 +822,7 @@ _ecore_evas_win32_cursor_set(Ecore_Evas *ee, Evas_Object *obj, int layer, int ho
 static void
 _ecore_evas_win32_focus_set(Ecore_Evas *ee, int on __UNUSED__)
 {
-   ecore_win32_window_focus_set((struct _Ecore_Win32_Window *)ee->prop.window);
+   ecore_win32_window_focus((struct _Ecore_Win32_Window *)ee->prop.window);
 }
 
 static void
@@ -804,6 +864,24 @@ _ecore_evas_win32_borderless_set(Ecore_Evas *ee, int on)
 #endif /* BUILD_ECORE_EVAS_SOFTWARE_GDI */
 }
 
+static void
+_ecore_evas_win32_override_set(Ecore_Evas *ee, int on)
+{
+   struct _Ecore_Win32_Window *window;
+
+   INF("ecore evas override set");
+
+   window = (struct _Ecore_Win32_Window *)ee->prop.window;
+
+   if (ee->prop.override == on) return;
+   if (ee->should_be_visible) ecore_win32_window_hide(window);
+   /* FIXME: use borderless_set for now */
+   ecore_win32_window_borderless_set(window, on);
+   if (ee->should_be_visible) ecore_win32_window_show(window);
+   if (ee->prop.focused) ecore_win32_window_focus(window);
+   ee->prop.override = on;
+}
+
 static void
 _ecore_evas_win32_fullscreen_set(Ecore_Evas *ee, int on)
 {
@@ -822,18 +900,11 @@ _ecore_evas_win32_fullscreen_set(Ecore_Evas *ee, int on)
 
    if (on != 0)
    {
-      ecore_win32_window_shape_set((struct _Ecore_Win32_Window *)ee->prop.window,
-                                   0, 0, NULL);
-      ecore_win32_window_fullscreen_set((struct _Ecore_Win32_Window *)ee->prop.window,
-                                        on);
+      ecore_win32_window_fullscreen_set(window, on);
    }
    else
    {
       ecore_win32_window_fullscreen_set(window, on);
-      ecore_win32_window_shape_set(window,
-                                   window->shape.width,
-                                   window->shape.height,
-                                   window->shape.mask);
    }
 
    /* Nothing to be done for the GDI backend at the evas level */
@@ -915,7 +986,7 @@ static Ecore_Evas_Engine_Func _ecore_win32_engine_func =
      _ecore_evas_win32_focus_set,
      _ecore_evas_win32_iconified_set,
      _ecore_evas_win32_borderless_set,
-     NULL, /* _ecore_evas_x_override_set */
+     _ecore_evas_win32_override_set,
      NULL,
      _ecore_evas_win32_fullscreen_set,
      NULL, /* _ecore_evas_x_avoid_damage_set */
@@ -924,8 +995,16 @@ static Ecore_Evas_Engine_Func _ecore_win32_engine_func =
      NULL, /* _ecore_evas_x_ignore_events_set */
      NULL,  /* _ecore_evas_x_alpha_set */
      NULL, //transparent
-     
-     NULL // render
+
+     NULL,
+     NULL,
+     NULL,
+     NULL,
+     NULL,
+     NULL,
+
+     NULL, // render
+     NULL  //screen_geometry_get
 };
 
 #endif /* BUILD_ECORE_EVAS_WIN32 */
@@ -962,8 +1041,14 @@ _ecore_evas_engine_software_gdi_init(Ecore_Evas *ee)
         if (!evas_engine_info_set(ee->evas, (Evas_Engine_Info *)einfo))
           {
              ERR("evas_engine_info_set() for engine '%s' failed.", ee->driver);
+             return 0;
           }
      }
+   else
+     {
+        ERR("evas_engine_info_set() init engine '%s' failed.", ee->driver);
+        return 0;
+     }
 
    return 1;
 }
@@ -996,8 +1081,14 @@ _ecore_evas_engine_software_ddraw_init(Ecore_Evas *ee)
         if (!evas_engine_info_set(ee->evas, (Evas_Engine_Info *)einfo))
           {
              ERR("evas_engine_info_set() for engine '%s' failed.", ee->driver);
+             return 0;
           }
      }
+   else
+     {
+        ERR("evas_engine_info_set() init engine '%s' failed.", ee->driver);
+        return 0;
+     }
 
    return 1;
 }
@@ -1030,8 +1121,14 @@ _ecore_evas_engine_direct3d_init(Ecore_Evas *ee)
         if (!evas_engine_info_set(ee->evas, (Evas_Engine_Info *)einfo))
           {
              ERR("evas_engine_info_set() for engine '%s' failed.", ee->driver);
+             return 0;
           }
      }
+   else
+     {
+        ERR("evas_engine_info_set() init engine '%s' failed.", ee->driver);
+        return 0;
+     }
 
    return 1;
 }
@@ -1063,8 +1160,14 @@ _ecore_evas_engine_opengl_glew_init(Ecore_Evas *ee)
         if (!evas_engine_info_set(ee->evas, (Evas_Engine_Info *)einfo))
           {
              ERR("evas_engine_info_set() for engine '%s' failed.", ee->driver);
+             return 0;
           }
      }
+   else
+     {
+        ERR("evas_engine_info_set() init engine '%s' failed.", ee->driver);
+        return 0;
+     }
 
    return 1;
 }
@@ -1100,8 +1203,14 @@ _ecore_evas_engine_software_16_ddraw_init(Ecore_Evas *ee)
         if (!evas_engine_info_set(ee->evas, (Evas_Engine_Info *)einfo))
           {
              ERR("evas_engine_info_set() for engine '%s' failed.", ee->driver);
+             return 0;
           }
      }
+   else
+     {
+        ERR("evas_engine_info_set() init engine '%s' failed.", ee->driver);
+        return 0;
+     }
 
    return 1;
 }
@@ -1137,6 +1246,10 @@ _ecore_evas_win32_new_internal(int (*_ecore_evas_engine_init)(Ecore_Evas *ee),
    ee->y = y;
    ee->w = width;
    ee->h = height;
+   ee->req.x = ee->x;
+   ee->req.y = ee->y;
+   ee->req.w = ee->w;
+   ee->req.h = ee->h;
 
    ee->prop.max.w = 32767;
    ee->prop.max.h = 32767;
@@ -1175,7 +1288,7 @@ _ecore_evas_win32_new_internal(int (*_ecore_evas_engine_init)(Ecore_Evas *ee),
                                (Ecore_Event_Multi_Move_Cb)_ecore_evas_mouse_multi_move_process,
                                (Ecore_Event_Multi_Down_Cb)_ecore_evas_mouse_multi_down_process,
                                (Ecore_Event_Multi_Up_Cb)_ecore_evas_mouse_multi_up_process);
-   
+
    return ee;
 }
 
