@@ -19,7 +19,7 @@
 
 #ifdef BUILD_ECORE_EVAS_SOFTWARE_16_WINCE
 
-#define ECORE_EVAS_EVENT_COUNT 7
+#define ECORE_EVAS_EVENT_COUNT 9
 
 static int _ecore_evas_init_count = 0;
 
@@ -29,6 +29,10 @@ static Eina_Bool _ecore_evas_wince_event_mouse_in(void *data __UNUSED__, int typ
 
 static Eina_Bool _ecore_evas_wince_event_mouse_out(void *data __UNUSED__, int type __UNUSED__, void *event);
 
+static Eina_Bool _ecore_evas_wince_event_window_focus_in(void *data __UNUSED__, int type __UNUSED__, void *event);
+
+static Eina_Bool _ecore_evas_wince_event_window_focus_out(void *data __UNUSED__, int type __UNUSED__, void *event);
+
 static Eina_Bool _ecore_evas_wince_event_window_damage(void *data __UNUSED__, int type __UNUSED__, void *event);
 
 static Eina_Bool _ecore_evas_wince_event_window_destroy(void *data __UNUSED__, int type __UNUSED__, void *event);
@@ -46,17 +50,17 @@ _ecore_evas_wince_render(Ecore_Evas *ee)
 {
    int rend = 0;
    Eina_List *updates = NULL;
-#ifdef BUILD_ECORE_EVAS_SOFTWARE_BUFFER
    Eina_List *ll;
    Ecore_Evas *ee2;
 
    EINA_LIST_FOREACH(ee->sub_ecore_evas, ll, ee2)
      {
         if (ee2->func.fn_pre_render) ee2->func.fn_pre_render(ee2);
-        rend |= _ecore_evas_buffer_render(ee2);
+        if (ee2->engine.func->fn_render)
+          rend |= ee2->engine.func->fn_render(ee2);
         if (ee2->func.fn_post_render) ee2->func.fn_post_render(ee2);
      }
-#endif
+
    if (ee->func.fn_pre_render) ee->func.fn_pre_render(ee);
    if (ee->prop.avoid_damage)
      {
@@ -91,14 +95,16 @@ _ecore_evas_wince_init(void)
    _ecore_evas_init_count++;
    if (_ecore_evas_init_count > 1)
      return _ecore_evas_init_count;
-   
+
    ecore_evas_event_handlers[0]  = ecore_event_handler_add(ECORE_WINCE_EVENT_MOUSE_IN, _ecore_evas_wince_event_mouse_in, NULL);
    ecore_evas_event_handlers[1]  = ecore_event_handler_add(ECORE_WINCE_EVENT_MOUSE_OUT, _ecore_evas_wince_event_mouse_out, NULL);
-   ecore_evas_event_handlers[2]  = ecore_event_handler_add(ECORE_WINCE_EVENT_WINDOW_DAMAGE, _ecore_evas_wince_event_window_damage, NULL);
-   ecore_evas_event_handlers[3]  = ecore_event_handler_add(ECORE_WINCE_EVENT_WINDOW_DESTROY, _ecore_evas_wince_event_window_destroy, NULL);
-   ecore_evas_event_handlers[4]  = ecore_event_handler_add(ECORE_WINCE_EVENT_WINDOW_SHOW, _ecore_evas_wince_event_window_show, NULL);
-   ecore_evas_event_handlers[5]  = ecore_event_handler_add(ECORE_WINCE_EVENT_WINDOW_HIDE, _ecore_evas_wince_event_window_hide, NULL);
-   ecore_evas_event_handlers[6]  = ecore_event_handler_add(ECORE_WINCE_EVENT_WINDOW_DELETE_REQUEST, _ecore_evas_wince_event_window_delete_request, NULL);
+   ecore_evas_event_handlers[2]  = ecore_event_handler_add(ECORE_WINCE_EVENT_WINDOW_FOCUS_IN, _ecore_evas_wince_event_window_focus_in, NULL);
+   ecore_evas_event_handlers[3]  = ecore_event_handler_add(ECORE_WINCE_EVENT_WINDOW_FOCUS_OUT, _ecore_evas_wince_event_window_focus_out, NULL);
+   ecore_evas_event_handlers[4]  = ecore_event_handler_add(ECORE_WINCE_EVENT_WINDOW_DAMAGE, _ecore_evas_wince_event_window_damage, NULL);
+   ecore_evas_event_handlers[5]  = ecore_event_handler_add(ECORE_WINCE_EVENT_WINDOW_DESTROY, _ecore_evas_wince_event_window_destroy, NULL);
+   ecore_evas_event_handlers[6]  = ecore_event_handler_add(ECORE_WINCE_EVENT_WINDOW_SHOW, _ecore_evas_wince_event_window_show, NULL);
+   ecore_evas_event_handlers[7]  = ecore_event_handler_add(ECORE_WINCE_EVENT_WINDOW_HIDE, _ecore_evas_wince_event_window_hide, NULL);
+   ecore_evas_event_handlers[8]  = ecore_event_handler_add(ECORE_WINCE_EVENT_WINDOW_DELETE_REQUEST, _ecore_evas_wince_event_window_delete_request, NULL);
 
    ecore_event_evas_init();
    return _ecore_evas_init_count;
@@ -168,6 +174,40 @@ _ecore_evas_wince_event_mouse_out(void *data __UNUSED__, int type __UNUSED__, vo
    return 1;
 }
 
+static Eina_Bool
+_ecore_evas_wince_event_window_focus_in(void *data __UNUSED__, int type __UNUSED__, void *event)
+{
+   Ecore_Evas *ee;
+   Ecore_WinCE_Event_Window_Focus_In *e;
+
+   e = event;
+   ee = ecore_event_window_match(e->window);
+   if ((!ee) || (ee->ignore_events)) return ECORE_CALLBACK_PASS_ON;
+   if (e->window != ee->prop.window) return ECORE_CALLBACK_PASS_ON;
+
+   ee->prop.focused = 1;
+   evas_focus_in(ee->evas);
+   if (ee->func.fn_focus_in) ee->func.fn_focus_in(ee);
+   return ECORE_CALLBACK_PASS_ON;
+}
+
+static Eina_Bool
+_ecore_evas_wince_event_window_focus_out(void *data __UNUSED__, int type __UNUSED__, void *event)
+{
+   Ecore_Evas *ee;
+   Ecore_WinCE_Event_Window_Focus_Out *e;
+
+   e = event;
+   ee = ecore_event_window_match(e->window);
+   if ((!ee) || (ee->ignore_events)) return ECORE_CALLBACK_PASS_ON;
+   if (e->window != ee->prop.window) return ECORE_CALLBACK_PASS_ON;
+
+   evas_focus_out(ee->evas);
+   ee->prop.focused = 0;
+   if (ee->func.fn_focus_out) ee->func.fn_focus_out(ee);
+   return ECORE_CALLBACK_PASS_ON;
+}
+
 static Eina_Bool
 _ecore_evas_wince_event_window_damage(void *data __UNUSED__, int type __UNUSED__, void *event)
 {
@@ -305,7 +345,7 @@ _ecore_evas_wince_free(Ecore_Evas *ee)
 
 static void
 _ecore_evas_wince_callback_delete_request_set(Ecore_Evas *ee,
-                                              void (*func) (Ecore_Evas *ee))
+                                              Ecore_Evas_Event_Cb func)
 {
    ee->func.fn_delete_request = func;
 }
@@ -314,6 +354,8 @@ static void
 _ecore_evas_wince_move(Ecore_Evas *ee, int x, int y)
 {
   INF("ecore evas move (%dx%d)", x, y);
+   ee->req.x = x;
+   ee->req.y = y;
 
    if ((x != ee->x) || (y != ee->y))
      {
@@ -328,6 +370,8 @@ static void
 _ecore_evas_wince_resize(Ecore_Evas *ee, int width, int height)
 {
    INF("ecore evas resize (%dx%d)", width, height);
+   ee->req.w = width;
+   ee->req.h = height;
 
    if ((ee->w != width) || (ee->h != height))
      {
@@ -354,6 +398,10 @@ static void
 _ecore_evas_wince_move_resize(Ecore_Evas *ee, int x, int y, int width, int height)
 {
    INF("ecore evas resize (%dx%d %dx%d)", x, y, width, height);
+   ee->req.x = x;
+   ee->req.y = y;
+   ee->req.w = width;
+   ee->req.h = height;
 
    if ((ee->w != width) || (ee->h != height) || (x != ee->x) || (y != ee->y))
      {
@@ -593,11 +641,11 @@ _ecore_evas_wince_cursor_set(Ecore_Evas *ee, Evas_Object *obj, int layer, int ho
 #endif
 }
 
-/* static void */
-/* _ecore_evas_wince_focus_set(Ecore_Evas *ee, int on __UNUSED__) */
-/* { */
-/*    ecore_wince_window_focus_set(ee->prop.window); */
-/* } */
+static void
+_ecore_evas_wince_focus_set(Ecore_Evas *ee, int on __UNUSED__)
+{
+   ecore_wince_window_focus(ee->prop.window);
+}
 
 /* static void */
 /* _ecore_evas_wince_iconified_set(Ecore_Evas *ee, int on) */
@@ -640,6 +688,8 @@ _ecore_evas_wince_fullscreen_set(Ecore_Evas *ee, int on)
       ecore_wince_window_fullscreen_set((Ecore_WinCE_Window *)ee->prop.window, on);
       ee->w = GetSystemMetrics(SM_CXSCREEN);
       ee->h = GetSystemMetrics(SM_CYSCREEN);
+      ee->req.w = ee->w;
+      ee->req.h = ee->h;
       evas_output_size_set(ee->evas, ee->w, ee->h);
       evas_output_viewport_set(ee->evas, 0, 0, ee->w, ee->h);
    }
@@ -652,6 +702,8 @@ _ecore_evas_wince_fullscreen_set(Ecore_Evas *ee, int on)
       ecore_wince_window_size_get((Ecore_WinCE_Window *)ee->prop.window, &w, &h);
       ee->w = w;
       ee->h = h;
+      ee->req.w = ee->w;
+      ee->req.h = ee->h;
       evas_output_size_set(ee->evas, ee->w, ee->h);
       evas_output_viewport_set(ee->evas, 0, 0, ee->w, ee->h);
 /*       ecore_win32_window_shape_set(window, */
@@ -708,7 +760,7 @@ static Ecore_Evas_Engine_Func _ecore_wince_engine_func =
    NULL, //_ecore_evas_wince_size_step_set,
    _ecore_evas_wince_cursor_set,
    NULL, /* _ecore_evas_x_layer_set */
-   NULL, //_ecore_evas_wince_focus_set,
+   _ecore_evas_wince_focus_set,
    NULL, //_ecore_evas_wince_iconified_set,
    NULL, //_ecore_evas_wince_borderless_set,
    NULL, /* _ecore_evas_x_override_set */
@@ -720,8 +772,16 @@ static Ecore_Evas_Engine_Func _ecore_wince_engine_func =
    NULL, /* _ecore_evas_x_ignore_events_set */
    NULL, /* _ecore_evas_x_alpha_set */
    NULL, //transparent
-     
-     NULL // render
+
+   NULL,
+   NULL,
+   NULL,
+   NULL,
+   NULL,
+   NULL,
+
+   NULL, // render
+   NULL  // screen_geometry_get
 };
 
 /* API */
@@ -772,6 +832,10 @@ ecore_evas_software_wince_new_internal(int                 backend,
    ee->y = y;
    ee->w = width;
    ee->h = height;
+   ee->req.x = ee->x;
+   ee->req.y = ee->y;
+   ee->req.w = ee->w;
+   ee->req.h = ee->h;
 
    ee->prop.max.w = 32767;
    ee->prop.max.h = 32767;
@@ -811,12 +875,24 @@ ecore_evas_software_wince_new_internal(int                 backend,
         if (!evas_engine_info_set(ee->evas, (Evas_Engine_Info *)einfo))
           {
              ERR("evas_engine_info_set() for engine '%s' failed.", ee->driver);
+             _ecore_evas_wince_shutdown();
+             free(ee);
+             ecore_wince_shutdown();
+             return NULL;
           }
 
         ecore_wince_window_backend_set((Ecore_WinCE_Window *)ee->prop.window, backend);
         ecore_wince_window_suspend_cb_set((Ecore_WinCE_Window *)ee->prop.window, einfo->func.suspend);
         ecore_wince_window_resume_cb_set((Ecore_WinCE_Window *)ee->prop.window, einfo->func.resume);
      }
+   else
+     {
+        ERR("evas_engine_info_set() init engine '%s' failed.", ee->driver);
+        _ecore_evas_wince_shutdown();
+        free(ee);
+        ecore_wince_shutdown();
+        return NULL;
+     }
 
    ee->engine.func->fn_render = _ecore_evas_wince_render;
    _ecore_evas_register(ee);
