@@ -2,6 +2,7 @@
 # include <config.h>
 #endif
 
+#include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 
@@ -10,12 +11,20 @@
 # include <libgen.h>
 #endif
 
+#ifdef _WIN32
+# include <direct.h>
+#endif
+
 #ifdef HAVE_FEATURES_H
 # include <features.h>
 #endif
 #include <ctype.h>
 #include <errno.h>
 
+#ifdef HAVE_ATFILE_SOURCE
+# include <dirent.h>
+#endif
+
 #include "ecore_file_private.h"
 
 int _ecore_file_log_dom = -1;
@@ -47,6 +56,10 @@ ecore_file_init()
 {
    if (++_ecore_file_init_count != 1)
      return _ecore_file_init_count;
+
+   if (!ecore_init())
+     return --_ecore_file_init_count;
+
    _ecore_file_log_dom = eina_log_domain_register
      ("ecore_file", ECORE_FILE_DEFAULT_LOG_COLOR);
    if(_ecore_file_log_dom < 0)
@@ -99,8 +112,12 @@ ecore_file_shutdown()
    ecore_file_download_shutdown();
    ecore_file_monitor_shutdown();
    ecore_file_path_shutdown();
+
    eina_log_domain_unregister(_ecore_file_log_dom);
    _ecore_file_log_dom = -1;
+
+   ecore_shutdown();
+
    return _ecore_file_init_count;
 }
 
@@ -145,15 +162,16 @@ ecore_file_size(const char *file)
  * @brief Check if the given file exists.
  *
  * @param file The name of the file.
- * @return Return EINA_TRUE if the file exists, EINA_FALSE otherwise.
+ * @return @c EINA_TRUE if the @p file exists, @c EINA_FALSE otherwise.
  *
- * This function returns EINA_TRUE if @p file exists on local filesystem,
- * EINA_FALSE otherwise.
+ * This function returns @c EINA_TRUE if @p file exists on local filesystem,
+ * @c EINA_FALSE otherwise.
  */
 EAPI Eina_Bool
 ecore_file_exists(const char *file)
 {
    struct stat st;
+   if (!file) return EINA_FALSE;
 
    /*Workaround so that "/" returns a true, otherwise we can't monitor "/" in ecore_file_monitor*/
    if (stat(file, &st) < 0 && strcmp(file, "/")) return EINA_FALSE;
@@ -164,11 +182,11 @@ ecore_file_exists(const char *file)
  * @brief Check if the given file is a directory.
  *
  * @param file The name of the file.
- * @return Return EINA_TRUE if the file exists and is a directory,
- * EINA_FALSE otherwise.
+ * @return @c EINA_TRUE if the file exists and is a directory, @c EINA_FALSE
+ * otherwise.
  *
- * This function returns EINA_TRUE if @p file exists exists and is a
- * directory on local filesystem, EINA_FALSE otherwise.
+ * This function returns @c EINA_TRUE if @p file exists exists and is a
+ * directory on local filesystem, @c EINA_FALSE otherwise.
  */
 EAPI Eina_Bool
 ecore_file_is_dir(const char *file)
@@ -186,11 +204,11 @@ static mode_t default_mode = S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S
  * @brief Create a new directory.
  *
  * @param  dir The name of the directory to create
- * @return EINA_TRUE on successful creation, EINA_FALSE otherwise.
+ * @return @c EINA_TRUE on successful creation, @c EINA_FALSE otherwise.
  *
  * This function creates the directory @p dir with the mode S_IRUSR |
  * S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH. On
- * success, it returns EINA_TRUE, EINA_FALSE otherwise.
+ * success, it returns @c EINA_TRUE, @c EINA_FALSE otherwise.
  */
 EAPI Eina_Bool
 ecore_file_mkdir(const char *dir)
@@ -241,7 +259,7 @@ ecore_file_mkdirs(const char **dirs)
  * returns -1 if @p dirs or @p base are @c NULL, or if @p base is
  * empty ("\0"). It returns 0 is @p base is not a directory or
  * invalid, or if it can't be created. Otherwise if returns the number
- * of suceesfully created directories. 
+ * of suceesfully created directories.
  */
 EAPI int
 ecore_file_mksubdirs(const char *base, const char **subdirs)
@@ -324,10 +342,10 @@ ecore_file_mksubdirs(const char *base, const char **subdirs)
  * @brief Delete the given directory.
  *
  * @param  dir The name of the directory to delete.
- * @return EINA_TRUE on success, EINA_FALSE otherwise.
+ * @return @c EINA_TRUE on success, @c EINA_FALSE otherwise.
  *
- * This function deletes @p dir. It returns EINA_TRUE on success,
- * EINA_FALSE otherwise.
+ * This function deletes @p dir. It returns @c EINA_TRUE on success,
+ * @c EINA_FALSE otherwise.
  */
 EAPI Eina_Bool
 ecore_file_rmdir(const char *dir)
@@ -340,10 +358,10 @@ ecore_file_rmdir(const char *dir)
  * @brief Delete the given file.
  *
  * @param  file The name of the file to delete.
- * @return EINA_TRUE on success, EINA_FALSE otherwise.
+ * @return @c EINA_TRUE on success, @c EINA_FALSE otherwise.
  *
- * This function deletes @p file. It returns EINA_TRUE on success,
- * EINA_FALSE otherwise.
+ * This function deletes @p file. It returns @c EINA_TRUE on success,
+ * @c EINA_FALSE otherwise.
  */
 EAPI Eina_Bool
 ecore_file_unlink(const char *file)
@@ -356,10 +374,10 @@ ecore_file_unlink(const char *file)
  * @brief Remove the given file or directory.
  *
  * @param  file The name of the file or directory to delete.
- * @return EINA_TRUE on success, EINA_FALSE otherwise.
+ * @return @c EINA_TRUE on success, @c EINA_FALSE otherwise.
  *
- * This function removes @p file. It returns EINA_TRUE on success,
- * EINA_FALSE otherwise.
+ * This function removes @p file. It returns @c EINA_TRUE on success,
+ * @c EINA_FALSE otherwise.
  */
 EAPI Eina_Bool
 ecore_file_remove(const char *file)
@@ -372,18 +390,17 @@ ecore_file_remove(const char *file)
  * @brief Delete the given directory and all its contents.
  *
  * @param  dir The name of the directory to delete.
- * @return EINA_TRUE on success, EINA_FALSE otherwise.
+ * @return @c EINA_TRUE on success, @c EINA_FALSE otherwise.
  *
  * This function delete @p dir and all its contents. If @p dir is a
- * link only the link is removed. It returns EINA_TRUE on success,
- * EINA_FALSE otherwise.
+ * link only the link is removed. It returns @c EINA_TRUE on success,
+ * @c EINA_FALSE otherwise.
  */
 EAPI Eina_Bool
 ecore_file_recursive_rm(const char *dir)
 {
-   DIR *dirp;
-   struct dirent *dp;
-   char path[PATH_MAX], buf[PATH_MAX];
+   Eina_Iterator *it;
+   char buf[PATH_MAX];
    struct stat st;
    int ret;
 
@@ -393,22 +410,19 @@ ecore_file_recursive_rm(const char *dir)
    ret = stat(dir, &st);
    if ((ret == 0) && (S_ISDIR(st.st_mode)))
      {
+        Eina_File_Direct_Info *info;
+
         ret = 1;
-        if (stat(dir, &st) == -1) return EINA_FALSE;
-        dirp = opendir(dir);
-        if (dirp)
+        if (stat(dir, &st) == -1) return EINA_FALSE; /* WOOT: WHY ARE WE CALLING STAT TWO TIMES ??? */
+
+        it = eina_file_direct_ls(dir);
+        EINA_ITERATOR_FOREACH(it, info)
           {
-             while ((dp = readdir(dirp)))
-               {
-                  if ((strcmp(dp->d_name, ".")) && (strcmp(dp->d_name, "..")))
-                    {
-                       snprintf(path, PATH_MAX, "%s/%s", dir, dp->d_name);
-                       if (!ecore_file_recursive_rm(path))
-                         ret = 0;
-                    }
-               }
-             closedir(dirp);
+             if (!ecore_file_recursive_rm(info->path))
+               ret = 0;
           }
+        eina_iterator_free(it);
+
         if (!ecore_file_rmdir(dir)) ret = 0;
         if (ret)
             return EINA_TRUE;
@@ -448,12 +462,12 @@ _ecore_file_mkpath_if_not_exists(const char *path)
  * @brief Create a complete path.
  *
  * @param  path The path to create
- * @return EINA_TRUE on success, EINA_FALSE otherwise.
+ * @return @c EINA_TRUE on success, @c EINA_FALSE otherwise.
  *
- * This function create @p path and all the subdirectories it
+ * This function creates @p path and all the subdirectories it
  * contains. The separator is '/' or '\'. If @p path exists, this
- * function returns EINA_TRUE immediatly. It returns EINA_TRUE on
- * success, EINA_FALSE otherwise.
+ * function returns @c EINA_TRUE immediately. It returns @c EINA_TRUE on
+ * success, @c EINA_FALSE otherwise.
  */
 EAPI Eina_Bool
 ecore_file_mkpath(const char *path)
@@ -489,7 +503,7 @@ ecore_file_mkpath(const char *path)
  * and call ecore_file_mkpath(), hence on Windows, '\' must be
  * replaced by '/' before calling that function. This function
  * returns -1 if @p paths is @c NULL. Otherwise if returns the number
- * of suceesfully created directories. 
+ * of suceesfully created directories.
  */
 EAPI int
 ecore_file_mkpaths(const char **paths)
@@ -509,12 +523,12 @@ ecore_file_mkpaths(const char **paths)
  *
  * @param  src The name of the source file.
  * @param  dst The name of the destination file.
- * @return EINA_TRUE on success, EINA_FALSE otherwise.
+ * @return @c EINA_TRUE on success, @c EINA_FALSE otherwise.
  *
  * This function copies @p src to @p dst. If the absolute path name of
  * @p src and @p dst can not be computed, or if they are equal, or if
- * the copy fails, the function returns EINA_FALSE, otherwise it
- * returns EINA_TRUE.
+ * the copy fails, the function returns @c EINA_FALSE, otherwise it
+ * returns @c EINA_TRUE.
  */
 EAPI Eina_Bool
 ecore_file_cp(const char *src, const char *dst)
@@ -550,10 +564,10 @@ ecore_file_cp(const char *src, const char *dst)
  *
  * @param  src The name of the source file.
  * @param  dst The name of the destination file.
- * @return EINA_TRUE on success, EINA_FALSE otherwise.
+ * @return @c EINA_TRUE on success, @c EINA_FALSE otherwise.
  *
- * This function moves @p src to @p dst. It returns EINA_TRUE on
- * success, EINA_FALSE otherwise.
+ * This function moves @p src to @p dst. It returns @c EINA_TRUE on
+ * success, @c EINA_FALSE otherwise.
  */
 EAPI Eina_Bool
 ecore_file_mv(const char *src, const char *dst)
@@ -577,18 +591,14 @@ ecore_file_mv(const char *src, const char *dst)
                   char *dir;
 
                   dir = ecore_file_dir_get(dst);
-                  // Since we can't directly rename, try to 
+                  // Since we can't directly rename, try to
                   // copy to temp file in the dst directory
                   // and then rename.
-                  snprintf(buf, sizeof(buf), "%s/.%s.tmp.XXXXXX", 
+                  snprintf(buf, sizeof(buf), "%s/.%s.tmp.XXXXXX",
                            dir, ecore_file_file_get(dst));
                   free(dir);
                   fd = mkstemp(buf);
-                  if (fd < 0)
-                    {
-                       perror("mkstemp");
-                       goto FAIL;
-                    }
+                  if (fd < 0) goto FAIL;
                   close(fd);
 
                   // Copy to temp file
@@ -628,11 +638,11 @@ FAIL:
  *
  * @param  src The name of the file to link.
  * @param  dest The name of link.
- * @return EINA_TRUE on success, EINA_FALSE otherwise.
+ * @return @c EINA_TRUE on success, @c EINA_FALSE otherwise.
  *
  * This function create the symbolic link @p dest of @p src. This
- * function does not work on Windows. It returns EINA_TRUE on success,
- * EINA_FALSE otherwise.
+ * function does not work on Windows. It returns @c EINA_TRUE on success,
+ * @c EINA_FALSE otherwise.
  */
 EAPI Eina_Bool
 ecore_file_symlink(const char *src, const char *dest)
@@ -717,9 +727,9 @@ ecore_file_dir_get(const char *file)
  * @brief Check if the given file can be read.
  *
  * @param  file The name of the file.
- * @return EINA_TRUE if the file is readable, EINA_FALSE otherwise.
+ * @return @c EINA_TRUE if the @p file is readable, @c EINA_FALSE otherwise.
  *
- * This function returns EINA_TRUE if @p file can be read, EINA_FALSE
+ * This function returns @c EINA_TRUE if @p file can be read, @c EINA_FALSE
  * otherwise.
  */
 EAPI Eina_Bool
@@ -734,9 +744,9 @@ ecore_file_can_read(const char *file)
  * @brief Check if the given file can be written.
  *
  * @param  file The name of the file.
- * @return EINA_TRUE if the file is writable, EINA_FALSE otherwise.
+ * @return @c EINA_TRUE if the @p file is writable, @c EINA_FALSE otherwise.
  *
- * This function returns EINA_TRUE if @p file can be written, EINA_FALSE
+ * This function returns @c EINA_TRUE if @p file can be written, @c EINA_FALSE
  * otherwise.
  */
 EAPI Eina_Bool
@@ -748,12 +758,13 @@ ecore_file_can_write(const char *file)
 }
 
 /**
- * @bbrief Check if the given file can be executed.
+ * @brief Check if the given file can be executed.
  *
  * @param  file The name of the file.
- * @return EINA_TRUE if the file can be executed, EINA_FALSE otherwise.
+ * @return @c EINA_TRUE if the @p file can be executed, @c EINA_FALSE
+ * otherwise.
  *
- * This function returns EINA_TRUE if @p file can be executed, EINA_FALSE
+ * This function returns @c EINA_TRUE if @p file can be executed, @c EINA_FALSE
  * otherwise.
  */
 EAPI Eina_Bool
@@ -767,7 +778,7 @@ ecore_file_can_exec(const char *file)
 /**
  * @brief Get the path pointed by the given link.
  *
- * @param  link The name of the link.
+ * @param  lnk The name of the link.
  * @return The path pointed by link or NULL.
  *
  * This function returns the path pointed by @p link as a newly
@@ -776,12 +787,12 @@ ecore_file_can_exec(const char *file)
  * returned value must be freed.
  */
 EAPI char *
-ecore_file_readlink(const char *link)
+ecore_file_readlink(const char *lnk)
 {
    char buf[PATH_MAX];
    int count;
 
-   if ((count = readlink(link, buf, sizeof(buf) - 1)) < 0) return NULL;
+   if ((count = readlink(lnk, buf, sizeof(buf) - 1)) < 0) return NULL;
    buf[count] = 0;
    return strdup(buf);
 }
@@ -806,23 +817,21 @@ ecore_file_readlink(const char *link)
 EAPI Eina_List *
 ecore_file_ls(const char *dir)
 {
-   char *f;
-   DIR *dirp;
-   struct dirent *dp;
+   Eina_File_Direct_Info *info;
+   Eina_Iterator *ls;
    Eina_List *list = NULL;
 
-   dirp = opendir(dir);
-   if (!dirp) return NULL;
+   ls = eina_file_direct_ls(dir);
+   if (!ls) return NULL;
 
-   while ((dp = readdir(dirp)))
+   EINA_ITERATOR_FOREACH(ls, info)
      {
-        if ((strcmp(dp->d_name, ".")) && (strcmp(dp->d_name, "..")))
-          {
-               f = strdup(dp->d_name);
-               list = eina_list_append(list, f);
-          }
+        char *f;
+
+        f = strdup(info->path + info->name_start);
+        list = eina_list_append(list, f);
      }
-   closedir(dirp);
+   eina_iterator_free(ls);
 
    list = eina_list_sort(list, eina_list_count(list), EINA_COMPARE_CB(strcoll));
 
@@ -833,11 +842,10 @@ ecore_file_ls(const char *dir)
  * @brief Return the executable from the given command.
  *
  * @param app The application command, with parameters.
- *
- * This function returns the executable from @p app as a newly
- * allocated string. Arguments are removed and escae characters are
- * handled. If @p app is @c NULL, or on failure, the function returns
- * @c NULL. When not needed anymore, the returned value must be freed.
+ * @return The executable from @p app as a newly allocated string. Arguments 
+ * are removed and escape characters are handled. If @p app is @c NULL, or 
+ * on failure, the function returns @c NULL. When not needed anymore, the 
+ * returned value must be freed.
  */
 EAPI char *
 ecore_file_app_exe_get(const char *app)
@@ -850,7 +858,7 @@ ecore_file_app_exe_get(const char *app)
 
    p = (char *)app;
 restart:
-   while ((*p) && (isspace(*p))) p++;
+   while ((*p) && (isspace((unsigned char)*p))) p++;
    exe1 = p;
    while (*p)
      {
@@ -870,7 +878,7 @@ restart:
                in_quot_sing = 1;
              else if (*p == '\"')
                in_quot_dbl = 1;
-             if ((isspace(*p)) && (!((p > app) && (p[-1] != '\\'))))
+             if ((isspace((unsigned char)*p)) && ((p <= app) || (p[-1] == '\\')))
                break;
           }
         p++;
@@ -965,7 +973,7 @@ restart:
                in_quot_sing = 1;
              else if (*p == '\"')
                in_quot_dbl = 1;
-             else if (isspace(*p))
+             else if (isspace((unsigned char)*p))
                {
                   if (restart)
                     goto restart;
@@ -1031,11 +1039,11 @@ ecore_file_escape_name(const char *filename)
 }
 
 /**
- * @bried Remove the extension from the given file name.
+ * @brief Remove the extension from the given file name.
  *
  * @param  path The name of the file.
  * @return A newly allocated string with the extension stripped out or
- * NULL on errors.
+ * @c NULL on errors.
  *
  * This function removes the extension from @p path and returns the
  * result as a newly allocated string. If @p path is @c NULL, or on
@@ -1070,32 +1078,29 @@ ecore_file_strip_ext(const char *path)
  * @brief Check if the given directory is empty.
  *
  * @param  dir The name of the directory to check.
- * @return 1 if directory is empty, 0 if it has at least one file or
- * -1 in case of errors.
+ * @return @c 1 if directory is empty, @c 0 if it has at least one file or
+ * @c -1 in case of errors.
  *
  * This functions checks if @p dir is empty. The '.' and '..' files
  * will be ignored. If @p dir is empty, 1 is returned, if it contains
- * at least 1 file, 0 is returned. On failure, -1 is returned.
+ * at least one file, @c 0 is returned. On failure, @c -1 is returned.
  */
 EAPI int
 ecore_file_dir_is_empty(const char *dir)
 {
-   DIR *dirp;
-   struct dirent *dp;
+   Eina_File_Direct_Info *info;
+   Eina_Iterator *it;
 
-   dirp = opendir(dir);
-   if (!dirp) return -1;
+   it = eina_file_direct_ls(dir);
+   if (!it) return -1;
 
-   while ((dp = readdir(dirp)))
+   EINA_ITERATOR_FOREACH(it, info)
      {
-        if ((strcmp(dp->d_name, ".")) && (strcmp(dp->d_name, "..")))
-          {
-             closedir(dirp);
-             return 0;
-          }
+        eina_iterator_free(it);
+        return 0;
      }
-   
-   closedir(dirp);
+
+   eina_iterator_free(it);
    return 1;
 }
 
