@@ -38,6 +38,24 @@ DEFINE_OLEGUID(IID_IUnknown,       0x00000000L, 0, 0);
 
 static int       _ecore_win32_init_count = 0;
 
+static void
+_ecore_win32_size_check(Ecore_Win32_Window *win, int w, int h, int *dx, int *dy)
+{
+   int minimal_width;
+   int minimal_height;
+
+   minimal_width = GetSystemMetrics(SM_CXMIN);
+   minimal_height = GetSystemMetrics(SM_CYMIN);
+   if ((w) < MAX(minimal_width, (int)win->min_width))
+     *dx = 0;
+   if ((w) > (int)win->max_width)
+     *dx = 0;
+   if ((h) < MAX(minimal_height, (int)win->min_height))
+     *dy = 0;
+   if ((h) > (int)win->max_height)
+     *dy = 0;
+}
+
 LRESULT CALLBACK
 _ecore_win32_window_procedure(HWND   window,
                               UINT   message,
@@ -45,7 +63,7 @@ _ecore_win32_window_procedure(HWND   window,
                               LPARAM data_param)
 {
    Ecore_Win32_Callback_Data *data;
-   POINTS                     pt;
+   POINTS                     point;
    DWORD                      coord;
 
    data = (Ecore_Win32_Callback_Data *)malloc(sizeof(Ecore_Win32_Callback_Data));
@@ -55,26 +73,82 @@ _ecore_win32_window_procedure(HWND   window,
    data->message = message;
    data->window_param = window_param;
    data->data_param = data_param;
-   data->time = GetMessageTime();
+   data->timestamp = GetMessageTime();
    coord = GetMessagePos();
-   pt = MAKEPOINTS(coord);
-   data->x = pt.x;
-   data->y = pt.y;
+   point = MAKEPOINTS(coord);
+   data->x = point.x;
+   data->y = point.y;
+   data->discard_ctrl = EINA_FALSE;
 
    switch (data->message)
      {
        /* Keyboard input notifications */
      case WM_KEYDOWN:
-       INF("keydown message");
+     case WM_SYSKEYDOWN:
+       if ((data->message == WM_KEYDOWN) &&
+           (data->window_param == VK_CONTROL) &&
+           ((HIWORD(data->data_param) & KF_EXTENDED) == 0))
+         {
+            /* Ctrl left key is pressed */
+            BOOL res;
+            MSG next_msg;
+
+            /*
+             * we check if the next message
+             * - is a WM_KEYDOWN
+             * - has the same timestamp than the Ctrl one
+             * - is the key press of the right Alt key
+             */
+            res = PeekMessage(&next_msg, data->window,
+                              WM_KEYDOWN, WM_KEYDOWN,
+                              PM_NOREMOVE);
+            if (res &&
+                (next_msg.wParam == VK_MENU) &&
+                (next_msg.time == data->timestamp) &&
+                (HIWORD(next_msg.lParam) & KF_EXTENDED))
+              {
+                 INF("discard left Ctrl key press (sent by AltGr key press)");
+                 data->discard_ctrl = EINA_TRUE;
+              }
+         }
        _ecore_win32_event_handle_key_press(data, 1);
        return 0;
      case WM_CHAR:
+     case WM_SYSCHAR:
        INF("char message");
        _ecore_win32_event_handle_key_press(data, 0);
        return 0;
      case WM_KEYUP:
+     case WM_SYSKEYUP:
        INF("keyup message");
-       _ecore_win32_event_handle_key_release(data, 1);
+       if ((data->window_param == VK_CONTROL) &&
+           ((HIWORD(data->data_param) & KF_EXTENDED) == 0))
+         {
+            /* Ctrl left key is pressed */
+            BOOL res;
+            MSG next_msg;
+
+            /*
+             * we check if the next message
+             * - is a WM_KEYUP or WM_SYSKEYUP
+             * - has the same timestamp than the Ctrl one
+             * - is the key release of the right Alt key
+             */
+            res = PeekMessage(&next_msg, data->window,
+                              WM_KEYUP, WM_SYSKEYUP,
+                              PM_NOREMOVE);
+            if (res &&
+                ((next_msg.message == WM_KEYUP) ||
+                 (next_msg.message == WM_SYSKEYUP)) &&
+                (next_msg.wParam == VK_MENU) &&
+                (next_msg.time == data->timestamp) &&
+                (HIWORD(next_msg.lParam) & KF_EXTENDED))
+              {
+                 INF("discard left Ctrl key release (sent by AltGr key release)");
+                 data->discard_ctrl = EINA_TRUE;
+              }
+         }
+       _ecore_win32_event_handle_key_release(data);
        return 0;
      case WM_SETFOCUS:
        INF("setfocus message");
@@ -87,6 +161,7 @@ _ecore_win32_window_procedure(HWND   window,
        /* Mouse input notifications */
      case WM_LBUTTONDOWN:
        INF("left button down message");
+       SetCapture(window);
        _ecore_win32_event_handle_button_press(data, 1);
        return 0;
      case WM_MBUTTONDOWN:
@@ -98,9 +173,22 @@ _ecore_win32_window_procedure(HWND   window,
        _ecore_win32_event_handle_button_press(data, 3);
        return 0;
      case WM_LBUTTONUP:
-       INF("left button up message");
-       _ecore_win32_event_handle_button_release(data, 1);
-       return 0;
+       {
+          Ecore_Win32_Window *w = NULL;
+
+          INF("left button up message");
+
+          ReleaseCapture();
+          w = (Ecore_Win32_Window *)GetWindowLongPtr(window, GWLP_USERDATA);
+          if (w->drag.dragging)
+            {
+               w->drag.dragging = 0;
+               return 0;
+            }
+
+          _ecore_win32_event_handle_button_release(data, 1);
+          return 0;
+       }
      case WM_MBUTTONUP:
        INF("middle button up message");
        _ecore_win32_event_handle_button_release(data, 2);
@@ -111,12 +199,165 @@ _ecore_win32_window_procedure(HWND   window,
        return 0;
      case WM_MOUSEMOVE:
        {
-          RECT                        rect;
-          struct _Ecore_Win32_Window *w = NULL;
+          RECT                rect;
+          Ecore_Win32_Window *w = NULL;
 
           INF("moue move message");
 
-          w = (struct _Ecore_Win32_Window *)GetWindowLongPtr(window, GWL_USERDATA);
+          w = (Ecore_Win32_Window *)GetWindowLongPtr(window, GWLP_USERDATA);
+
+          if (w->drag.dragging)
+            {
+               POINT pt;
+
+               pt.x = GET_X_LPARAM(data_param);
+               pt.y = GET_Y_LPARAM(data_param);
+               if (ClientToScreen(window, &pt))
+                 {
+                    if (w->drag.type == HTCAPTION)
+                      {
+                         int dx;
+                         int dy;
+
+                         dx = pt.x - w->drag.px;
+                         dy = pt.y - w->drag.py;
+                         ecore_win32_window_move(w, w->drag.x + dx, w->drag.y + dy);
+                         w->drag.x += dx;
+                         w->drag.y += dy;
+                         w->drag.px = pt.x;
+                         w->drag.py = pt.y;
+                         return 0;
+                      }
+                    if (w->drag.type == HTLEFT)
+                      {
+                         int dw;
+
+                         dw = pt.x - w->drag.px;
+                         ecore_win32_window_move_resize(w, w->drag.x + dw, w->drag.y, w->drag.w - dw, w->drag.h);
+                         w->drag.x += dw;
+                         w->drag.w -= dw;
+                         w->drag.px = pt.x;
+                         w->drag.py = pt.y;
+                         return 0;
+                      }
+                    if (w->drag.type == HTRIGHT)
+                      {
+                         int dw;
+
+                         dw = pt.x - w->drag.px;
+                         ecore_win32_window_resize(w, w->drag.w + dw, w->drag.h);
+                         w->drag.w += dw;
+                         w->drag.px = pt.x;
+                         w->drag.py = pt.y;
+                         return 0;
+                      }
+                    if (w->drag.type == HTTOP)
+                      {
+                         int dh;
+
+                         dh = pt.y - w->drag.py;
+                         ecore_win32_window_move_resize(w, w->drag.x, w->drag.y + dh, w->drag.w, w->drag.h - dh);
+                         w->drag.y += dh;
+                         w->drag.h -= dh;
+                         w->drag.px = pt.x;
+                         w->drag.py = pt.y;
+                         return 0;
+                      }
+                    if (w->drag.type == HTBOTTOM)
+                      {
+                         int dh;
+
+                         dh = pt.y - w->drag.py;
+                         ecore_win32_window_resize(w, w->drag.w, w->drag.h + dh);
+                         w->drag.h += dh;
+                         w->drag.px = pt.x;
+                         w->drag.py = pt.y;
+                         return 0;
+                      }
+                    if (w->drag.type == HTTOPLEFT)
+                      {
+                         int dx;
+                         int dy;
+                         int dh;
+                         int dw;
+
+                         dw = pt.x - w->drag.px;
+                         dh = pt.y - w->drag.py;
+                         dx = dw;
+                         dy = dh;
+                         _ecore_win32_size_check(w,
+                                                 w->drag.w - dw, w->drag.h - dh,
+                                                 &dx, &dy);
+
+                         ecore_win32_window_move_resize(w, w->drag.x + dx, w->drag.y + dy, w->drag.w - dw, w->drag.h - dh);
+                         w->drag.x += dx;
+                         w->drag.y += dy;
+                         w->drag.w -= dw;
+                         w->drag.h -= dh;
+                         w->drag.px = pt.x;
+                         w->drag.py = pt.y;
+                         return 0;
+                      }
+                    if (w->drag.type == HTTOPRIGHT)
+                      {
+                         int dx;
+                         int dy;
+                         int dh;
+                         int dw;
+
+                         dw = pt.x - w->drag.px;
+                         dh = pt.y - w->drag.py;
+                         dx = dw;
+                         dy = dh;
+                         _ecore_win32_size_check(w,
+                                                 w->drag.w, w->drag.h - dh,
+                                                 &dx, &dy);
+                         ecore_win32_window_move_resize(w, w->drag.x, w->drag.y + dy, w->drag.w, w->drag.h - dh);
+                         w->drag.y += dy;
+                         w->drag.w += dw;
+                         w->drag.h -= dh;
+                         w->drag.px = pt.x;
+                         w->drag.py = pt.y;
+                         return 0;
+                      }
+                    if (w->drag.type == HTBOTTOMLEFT)
+                      {
+                         int dx;
+                         int dy;
+                         int dh;
+                         int dw;
+
+                         dw = pt.x - w->drag.px;
+                         dh = pt.y - w->drag.py;
+                         dx = dw;
+                         dy = dh;
+                         _ecore_win32_size_check(w,
+                                                 w->drag.w - dw, w->drag.h + dh,
+                                                 &dx, &dy);
+                         ecore_win32_window_move_resize(w, w->drag.x + dx, w->drag.y, w->drag.w - dw, w->drag.h + dh);
+                         w->drag.x += dx;
+                         w->drag.w -= dw;
+                         w->drag.h += dh;
+                         w->drag.px = pt.x;
+                         w->drag.py = pt.y;
+                         return 0;
+                      }
+                    if (w->drag.type == HTBOTTOMRIGHT)
+                      {
+                         int dh;
+                         int dw;
+
+                         dw = pt.x - w->drag.px;
+                         dh = pt.y - w->drag.py;
+                         ecore_win32_window_resize(w, w->drag.w + dw, w->drag.h + dh);
+                         w->drag.w += dw;
+                         w->drag.h += dh;
+                         w->drag.px = pt.x;
+                         w->drag.py = pt.y;
+                         return 0;
+                      }
+                 }
+            }
 
           if (GetClientRect(window, &rect))
             {
@@ -210,6 +451,7 @@ _ecore_win32_window_procedure(HWND   window,
      case WM_WINDOWPOSCHANGED:
        INF("position changed window message");
        _ecore_win32_event_handle_configure_notify(data);
+       _ecore_win32_event_handle_expose(data);
        return 0;
      case WM_ENTERSIZEMOVE:
        INF("enter size move window message");
@@ -219,6 +461,7 @@ _ecore_win32_window_procedure(HWND   window,
        return 0;
      case WM_NCLBUTTONDOWN:
        INF("non client left button down window message");
+
        if (((DWORD)window_param == HTCAPTION) ||
            ((DWORD)window_param == HTBOTTOM) ||
            ((DWORD)window_param == HTBOTTOMLEFT) ||
@@ -231,10 +474,12 @@ _ecore_win32_window_procedure(HWND   window,
          {
            Ecore_Win32_Window *w;
 
-           w = (Ecore_Win32_Window *)GetWindowLongPtr(window, GWL_USERDATA);
+           w = (Ecore_Win32_Window *)GetWindowLongPtr(window, GWLP_USERDATA);
            ecore_win32_window_geometry_get(w,
-                                           &w->drag.x, &w->drag.y,
+                                           NULL, NULL,
                                            &w->drag.w, &w->drag.h);
+           SetCapture(window);
+           w->drag.type = (DWORD)window_param;
            w->drag.px = GET_X_LPARAM(data_param);
            w->drag.py = GET_Y_LPARAM(data_param);
            w->drag.dragging = 1;
@@ -242,7 +487,8 @@ _ecore_win32_window_procedure(HWND   window,
          }
        return DefWindowProc(window, message, window_param, data_param);
      case WM_SYSCOMMAND:
-       INF("sys command window message", (int)window_param);
+       INF("sys command window message %d", (int)window_param);
+
        if ((((DWORD)window_param & 0xfff0) == SC_MOVE) ||
            (((DWORD)window_param & 0xfff0) == SC_SIZE))
          {
@@ -250,123 +496,14 @@ _ecore_win32_window_procedure(HWND   window,
 
            INF("sys command MOVE or SIZE window message : %dx%d", GET_X_LPARAM(data_param), GET_Y_LPARAM(data_param));
 
-           w = (Ecore_Win32_Window *)GetWindowLongPtr(window, GWL_USERDATA);
+           w = (Ecore_Win32_Window *)GetWindowLongPtr(window, GWLP_USERDATA);
            w->drag.dragging = 1;
            return 0;
          }
        return DefWindowProc(window, message, window_param, data_param);
-     case WM_NCMOUSEMOVE:
-       INF("non client mouse move window message");
-       if ((DWORD)window_param == HTCAPTION)
-         {
-           Ecore_Win32_Window *w;
-
-           w = (Ecore_Win32_Window *)GetWindowLongPtr(window, GWL_USERDATA);
-           if (w->drag.dragging)
-             {
-               int dx;
-               int dy;
-
-               dx = GET_X_LPARAM(data_param) - w->drag.px;
-               dy = GET_Y_LPARAM(data_param) - w->drag.py;
-               ecore_win32_window_move(w, w->drag.x + dx, w->drag.y + dy);
-               w->drag.x += dx;
-               w->drag.y += dy;
-               w->drag.px = GET_X_LPARAM(data_param);
-               w->drag.py = GET_Y_LPARAM(data_param);
-               return 0;
-             }
-         }
-       if ((DWORD)window_param == HTLEFT)
-         {
-           Ecore_Win32_Window *w;
-
-           w = (Ecore_Win32_Window *)GetWindowLongPtr(window, GWL_USERDATA);
-           if (w->drag.dragging)
-             {
-               int dw;
-               dw = GET_X_LPARAM(data_param) - w->drag.px;
-               ecore_win32_window_move_resize(w, w->drag.x + dw, w->drag.y, w->drag.w - dw, w->drag.h);
-               w->drag.x += dw;
-               w->drag.w -= dw;
-               w->drag.px = GET_X_LPARAM(data_param);
-               w->drag.py = GET_Y_LPARAM(data_param);
-               return 0;
-             }
-         }
-       if ((DWORD)window_param == HTRIGHT)
-         {
-           Ecore_Win32_Window *w;
-
-           w = (Ecore_Win32_Window *)GetWindowLongPtr(window, GWL_USERDATA);
-           if (w->drag.dragging)
-             {
-               int dw;
-               dw = GET_X_LPARAM(data_param) - w->drag.px;
-               ecore_win32_window_resize(w, w->drag.w + dw, w->drag.h);
-               w->drag.w += dw;
-               w->drag.px = GET_X_LPARAM(data_param);
-               w->drag.py = GET_Y_LPARAM(data_param);
-               return 0;
-             }
-         }
-       if ((DWORD)window_param == HTTOP)
-         {
-           Ecore_Win32_Window *w;
-
-           w = (Ecore_Win32_Window *)GetWindowLongPtr(window, GWL_USERDATA);
-           if (w->drag.dragging)
-             {
-               int dh;
-               dh = GET_Y_LPARAM(data_param) - w->drag.py;
-               ecore_win32_window_move_resize(w, w->drag.x, w->drag.y + dh, w->drag.w, w->drag.h - dh);
-               w->drag.y += dh;
-               w->drag.h -= dh;
-               w->drag.px = GET_X_LPARAM(data_param);
-               w->drag.py = GET_Y_LPARAM(data_param);
-               return 0;
-             }
-         }
-       if ((DWORD)window_param == HTBOTTOM)
-         {
-           Ecore_Win32_Window *w;
-
-           w = (Ecore_Win32_Window *)GetWindowLongPtr(window, GWL_USERDATA);
-           if (w->drag.dragging)
-             {
-               int dh;
-               dh = GET_Y_LPARAM(data_param) - w->drag.py;
-               ecore_win32_window_resize(w, w->drag.w, w->drag.h + dh);
-               w->drag.h += dh;
-               w->drag.px = GET_X_LPARAM(data_param);
-               w->drag.py = GET_Y_LPARAM(data_param);
-               return 0;
-             }
-         }
-       return DefWindowProc(window, message, window_param, data_param);
-     case WM_NCLBUTTONUP:
-       INF("non client left button up window message");
-       if (((DWORD)window_param == HTCAPTION) ||
-           ((DWORD)window_param == HTBOTTOM) ||
-           ((DWORD)window_param == HTBOTTOMLEFT) ||
-           ((DWORD)window_param == HTBOTTOMRIGHT) ||
-           ((DWORD)window_param == HTLEFT) ||
-           ((DWORD)window_param == HTRIGHT) ||
-           ((DWORD)window_param == HTTOP) ||
-           ((DWORD)window_param == HTTOPLEFT) ||
-           ((DWORD)window_param == HTTOPRIGHT))
-         {
-           Ecore_Win32_Window *w;
-
-           w = (Ecore_Win32_Window *)GetWindowLongPtr(window, GWL_USERDATA);
-           if (w->drag.dragging)
-             {
-               w->drag.dragging = 0;
-               return 0;
-             }
-         }
-       return DefWindowProc(window, message, window_param, data_param);
        /* GDI notifications */
+     case WM_ERASEBKGND:
+       return 1;
      case WM_PAINT:
        {
          RECT rect;
@@ -408,7 +545,7 @@ _ecore_win32_window_procedure(HWND   window,
 
 HINSTANCE           _ecore_win32_instance = NULL;
 double              _ecore_win32_double_click_time = 0.25;
-long                _ecore_win32_event_last_time = 0;
+unsigned long       _ecore_win32_event_last_time = 0;
 Ecore_Win32_Window *_ecore_win32_event_last_window = NULL;
 int                 _ecore_win32_log_dom_global = -1;
 
@@ -693,7 +830,7 @@ ecore_win32_double_click_time_get(void)
  *
  * This function returns the last event time.
  */
-EAPI long
+EAPI unsigned long
 ecore_win32_current_time_get(void)
 {
    return _ecore_win32_event_last_time;
