@@ -16,7 +16,7 @@
 # ifdef  __cplusplus
 extern "C"
 # endif /* ifdef  __cplusplus */
-void *    alloca (size_t);
+void *alloca(size_t);
 #endif /* ifdef HAVE_ALLOCA_H */
 
 #include <string.h>
@@ -270,7 +270,20 @@ _ecore_x_atoms_init(void)
       { "_E_ILLUME_QUICKPANEL_ZONE", &ECORE_X_ATOM_E_ILLUME_QUICKPANEL_ZONE },
       { "_E_ILLUME_QUICKPANEL_POSITION_UPDATE",
         &ECORE_X_ATOM_E_ILLUME_QUICKPANEL_POSITION_UPDATE },
-
+      { "_E_ILLUME_INDICATOR_STATE", &ECORE_X_ATOM_E_ILLUME_INDICATOR_STATE },
+      { "_E_ILLUME_INDICATOR_ON", &ECORE_X_ATOM_E_ILLUME_INDICATOR_ON },
+      { "_E_ILLUME_INDICATOR_OFF", &ECORE_X_ATOM_E_ILLUME_INDICATOR_OFF },
+      { "_E_ILLUME_INDICATOR_OPACITY_MODE", &ECORE_X_ATOM_E_ILLUME_INDICATOR_OPACITY_MODE },
+      { "_E_ILLUME_INDICATOR_OPAQUE", &ECORE_X_ATOM_E_ILLUME_INDICATOR_OPAQUE },
+      { "_E_ILLUME_INDICATOR_TRANSLUCENT", &ECORE_X_ATOM_E_ILLUME_INDICATOR_TRANSLUCENT },
+      { "_E_ILLUME_INDICATOR_TRANSPARENT", &ECORE_X_ATOM_E_ILLUME_INDICATOR_TRANSPARENT },
+      { "_E_ILLUME_ROTATE_WINDOW_AVAILABLE_ANGLES", &ECORE_X_ATOM_E_ILLUME_ROTATE_WINDOW_AVAILABLE_ANGLE },
+      { "_E_ILLUME_ROTATE_WINDOW_ANGLE", &ECORE_X_ATOM_E_ILLUME_ROTATE_WINDOW_ANGLE },
+      { "_E_ILLUME_ROTATE_ROOT_ANGLE", &ECORE_X_ATOM_E_ILLUME_ROTATE_ROOT_ANGLE },
+      { "_E_ILLUME_CLIPBOARD_STATE", &ECORE_X_ATOM_E_ILLUME_CLIPBOARD_STATE },
+      { "_E_ILLUME_CLIPBOARD_ON", &ECORE_X_ATOM_E_ILLUME_CLIPBOARD_ON },
+      { "_E_ILLUME_CLIPBOARD_OFF", &ECORE_X_ATOM_E_ILLUME_CLIPBOARD_OFF },
+      { "_E_ILLUME_CLIPBOARD_GEOMETRY", &ECORE_X_ATOM_E_ILLUME_CLIPBOARD_GEOMETRY },
       { "_E_COMP_SYNC_COUNTER", &ECORE_X_ATOM_E_COMP_SYNC_COUNTER },
       { "_E_COMP_SYNC_DRAW_DONE", &ECORE_X_ATOM_E_COMP_SYNC_DRAW_DONE },
       { "_E_COMP_SYNC_SUPPORTED", &ECORE_X_ATOM_E_COMP_SYNC_SUPPORTED },
@@ -280,7 +293,9 @@ _ecore_x_atoms_init(void)
 
       { "_E_COMP_FLUSH", &ECORE_X_ATOM_E_COMP_FLUSH },
       { "_E_COMP_DUMP", &ECORE_X_ATOM_E_COMP_DUMP },
-      { "_E_COMP_PIXMAP", &ECORE_X_ATOM_E_COMP_PIXMAP }
+      { "_E_COMP_PIXMAP", &ECORE_X_ATOM_E_COMP_PIXMAP },
+      { "_E_VIDEO_PARENT", &ECORE_X_ATOM_E_VIDEO_PARENT },
+      { "_E_VIDEO_POSITION", &ECORE_X_ATOM_E_VIDEO_POSITION }
    };
    Atom *atoms;
    char **names;
@@ -289,10 +304,12 @@ _ecore_x_atoms_init(void)
    num = sizeof(items) / sizeof(Atom_Item);
    atoms = alloca(num * sizeof(Atom));
    names = alloca(num * sizeof(char *));
-   for (i = 0; i < num; i++) names[i] = (char *)items[i].name;
+   for (i = 0; i < num; i++)
+     names[i] = (char *)items[i].name;
    XInternAtoms(_ecore_x_disp, names, num, False, atoms);
-   for (i = 0; i < num; i++) *(items[i].atom) = atoms[i];
-} /* _ecore_x_atoms_init */
+   for (i = 0; i < num; i++)
+     *(items[i].atom) = atoms[i];
+}
 
 /**
  * Retrieves the atom value associated with the given name.
@@ -303,27 +320,29 @@ EAPI Ecore_X_Atom
 ecore_x_atom_get(const char *name)
 {
    if (!_ecore_x_disp)
-      return 0;
+     return 0;
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    return XInternAtom(_ecore_x_disp, name, False);
-} /* ecore_x_atom_get */
+}
 
 EAPI void
-ecore_x_atoms_get(const char **names, int num, Ecore_X_Atom *atoms)
+ecore_x_atoms_get(const char **names,
+                  int num,
+                  Ecore_X_Atom *atoms)
 {
    Atom *atoms_int;
    int i;
 
    if (!_ecore_x_disp)
-      return;
+     return;
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    atoms_int = alloca(num * sizeof(Atom));
    XInternAtoms(_ecore_x_disp, (char **)names, num, False, atoms_int);
    for (i = 0; i < num; i++)
-      atoms[i] = atoms_int[i];
-} /* ecore_x_atoms_get */
+     atoms[i] = atoms_int[i];
+}
 
 EAPI char *
 ecore_x_atom_name_get(Ecore_X_Atom atom)
@@ -332,17 +351,17 @@ ecore_x_atom_name_get(Ecore_X_Atom atom)
    char *xname;
 
    if (!_ecore_x_disp)
-      return NULL;
+     return NULL;
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
 
    xname = XGetAtomName(_ecore_x_disp, atom);
    if (!xname)
-      return NULL;
+     return NULL;
 
    name = strdup(xname);
    XFree(xname);
 
    return name;
-} /* ecore_x_atom_name_get */
+}
 
