@@ -12,7 +12,7 @@
 
 /**
  * Retrieves the geometry of the given drawable.
-* @param d The given drawable.
+ * @param d The given drawable.
  * @param x Pointer to an integer into which the X position is to be stored.
  * @param y Pointer to an integer into which the Y position is to be stored.
  * @param w Pointer to an integer into which the width is to be stored.
@@ -21,10 +21,10 @@
  */
 EAPI void
 ecore_x_drawable_geometry_get(Ecore_X_Drawable d,
-                              int             *x,
-                              int             *y,
-                              int             *w,
-                              int             *h)
+                              int *x,
+                              int *y,
+                              int *w,
+                              int *h)
 {
    Window dummy_win;
    int ret_x, ret_y;
@@ -41,17 +41,17 @@ ecore_x_drawable_geometry_get(Ecore_X_Drawable d,
      }
 
    if (x)
-      *x = ret_x;
+     *x = ret_x;
 
    if (y)
-      *y = ret_y;
+     *y = ret_y;
 
    if (w)
-      *w = (int)ret_w;
+     *w = (int)ret_w;
 
    if (h)
-      *h = (int)ret_h;
-} /* ecore_x_drawable_geometry_get */
+     *h = (int)ret_h;
+}
 
 /**
  * Retrieves the width of the border of the given drawable.
@@ -69,10 +69,10 @@ ecore_x_drawable_border_width_get(Ecore_X_Drawable d)
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (!XGetGeometry(_ecore_x_disp, d, &dummy_win, &dummy_x, &dummy_y,
                      &dummy_w, &dummy_h, &border_ret, &dummy_depth))
-      border_ret = 0;
+     border_ret = 0;
 
    return (int)border_ret;
-} /* ecore_x_drawable_border_width_get */
+}
 
 /**
  * Retrieves the depth of the given drawable.
@@ -90,10 +90,10 @@ ecore_x_drawable_depth_get(Ecore_X_Drawable d)
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (!XGetGeometry(_ecore_x_disp, d, &dummy_win, &dummy_x, &dummy_y,
                      &dummy_w, &dummy_h, &dummy_border, &depth_ret))
-      depth_ret = 0;
+     depth_ret = 0;
 
    return (int)depth_ret;
-} /* ecore_x_drawable_depth_get */
+}
 
 /**
  * Fill the specified rectangle on a drawable.
@@ -106,13 +106,13 @@ ecore_x_drawable_depth_get(Ecore_X_Drawable d)
  */
 EAPI void
 ecore_x_drawable_rectangle_fill(Ecore_X_Drawable d,
-                                Ecore_X_GC       gc,
-                                int              x,
-                                int              y,
-                                int              width,
-                                int              height)
+                                Ecore_X_GC gc,
+                                int x,
+                                int y,
+                                int width,
+                                int height)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XFillRectangle(_ecore_x_disp, d, gc, x, y, width, height);
-} /* ecore_x_drawable_rectangle_fill */
+}
 
