@@ -17,7 +17,7 @@ EAPI Eina_Bool
 ecore_x_screensaver_event_available_get(void)
 {
    if (_screensaver_available >= 0)
-      return _screensaver_available;
+     return _screensaver_available;
 
 #ifdef ECORE_XSS
    int _screensaver_major, _screensaver_minor;
@@ -28,15 +28,15 @@ ecore_x_screensaver_event_available_get(void)
 
    if (XScreenSaverQueryVersion(_ecore_x_disp, &_screensaver_major,
                                 &_screensaver_minor))
-      _screensaver_available = 1;
+     _screensaver_available = 1;
    else
-      _screensaver_available = 0;
+     _screensaver_available = 0;
 
 #else /* ifdef ECORE_XSS */
    _screensaver_available = 0;
 #endif /* ifdef ECORE_XSS */
    return _screensaver_available;
-} /* ecore_x_screensaver_event_available_get */
+}
 
 EAPI int
 ecore_x_screensaver_idle_time_get(void)
@@ -49,15 +49,15 @@ ecore_x_screensaver_idle_time_get(void)
    xss = XScreenSaverAllocInfo();
    XScreenSaverQueryInfo(_ecore_x_disp,
                          RootWindow(_ecore_x_disp, DefaultScreen(
-                                       _ecore_x_disp)), xss);
+                                      _ecore_x_disp)), xss);
    idle = xss->idle / 1000;
    XFree(xss);
 
    return idle;
-#else   
+#else
    return 0;
 #endif /* ifdef ECORE_XSS */
-} /* ecore_x_screensaver_idle_time_get */
+}
 
 EAPI void
 ecore_x_screensaver_set(int timeout,
@@ -71,7 +71,7 @@ ecore_x_screensaver_set(int timeout,
                    interval,
                    prefer_blanking,
                    allow_exposures);
-} /* ecore_x_screensaver_set */
+}
 
 EAPI void
 ecore_x_screensaver_timeout_set(int timeout)
@@ -81,7 +81,7 @@ ecore_x_screensaver_timeout_set(int timeout)
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XGetScreenSaver(_ecore_x_disp, &pto, &pint, &pblank, &pexpo);
    XSetScreenSaver(_ecore_x_disp, timeout, pint, pblank, pexpo);
-} /* ecore_x_screensaver_timeout_set */
+}
 
 EAPI int
 ecore_x_screensaver_timeout_get(void)
@@ -91,7 +91,7 @@ ecore_x_screensaver_timeout_get(void)
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XGetScreenSaver(_ecore_x_disp, &pto, &pint, &pblank, &pexpo);
    return pto;
-} /* ecore_x_screensaver_timeout_get */
+}
 
 EAPI void
 ecore_x_screensaver_blank_set(int blank)
@@ -101,7 +101,7 @@ ecore_x_screensaver_blank_set(int blank)
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XGetScreenSaver(_ecore_x_disp, &pto, &pint, &pblank, &pexpo);
    XSetScreenSaver(_ecore_x_disp, pto, pint, blank, pexpo);
-} /* ecore_x_screensaver_blank_set */
+}
 
 EAPI int
 ecore_x_screensaver_blank_get(void)
@@ -111,7 +111,7 @@ ecore_x_screensaver_blank_get(void)
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XGetScreenSaver(_ecore_x_disp, &pto, &pint, &pblank, &pexpo);
    return pblank;
-} /* ecore_x_screensaver_blank_get */
+}
 
 EAPI void
 ecore_x_screensaver_expose_set(int expose)
@@ -121,7 +121,7 @@ ecore_x_screensaver_expose_set(int expose)
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XGetScreenSaver(_ecore_x_disp, &pto, &pint, &pblank, &pexpo);
    XSetScreenSaver(_ecore_x_disp, pto, pint, pblank, expose);
-} /* ecore_x_screensaver_expose_set */
+}
 
 EAPI int
 ecore_x_screensaver_expose_get(void)
@@ -131,7 +131,7 @@ ecore_x_screensaver_expose_get(void)
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XGetScreenSaver(_ecore_x_disp, &pto, &pint, &pblank, &pexpo);
    return pexpo;
-} /* ecore_x_screensaver_expose_get */
+}
 
 EAPI void
 ecore_x_screensaver_interval_set(int interval)
@@ -141,7 +141,7 @@ ecore_x_screensaver_interval_set(int interval)
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XGetScreenSaver(_ecore_x_disp, &pto, &pint, &pblank, &pexpo);
    XSetScreenSaver(_ecore_x_disp, pto, interval, pblank, pexpo);
-} /* ecore_x_screensaver_interval_set */
+}
 
 EAPI int
 ecore_x_screensaver_interval_get(void)
@@ -151,7 +151,7 @@ ecore_x_screensaver_interval_get(void)
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XGetScreenSaver(_ecore_x_disp, &pto, &pint, &pblank, &pexpo);
    return pint;
-} /* ecore_x_screensaver_interval_get */
+}
 
 EAPI void
 ecore_x_screensaver_event_listen_set(Eina_Bool on)
@@ -162,12 +162,12 @@ ecore_x_screensaver_event_listen_set(Eina_Bool on)
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    root = DefaultRootWindow(_ecore_x_disp);
    if (on)
-      XScreenSaverSelectInput(_ecore_x_disp, root, ScreenSaverNotifyMask);
+     XScreenSaverSelectInput(_ecore_x_disp, root, ScreenSaverNotifyMask);
    else
-      XScreenSaverSelectInput(_ecore_x_disp, root, 0);
+     XScreenSaverSelectInput(_ecore_x_disp, root, 0);
 #else
    return;
    on = EINA_FALSE;
 #endif /* ifdef ECORE_XSS */
-} /* ecore_x_screensaver_event_listen_set */
+}
 
