@@ -15,29 +15,29 @@ static Ecore_X_Selection_Intern selections[4];
 static Ecore_X_Selection_Converter *converters = NULL;
 static Ecore_X_Selection_Parser *parsers = NULL;
 
-static Eina_Bool _ecore_x_selection_converter_text(char         *target,
-                                                  void         *data,
-                                                  int           size,
-                                                  void        **data_ret,
-                                                  int          *size_ret,
-                                                  Ecore_X_Atom *tprop,
-                                                  int *);
-static int       _ecore_x_selection_data_default_free(void *data);
-static void *    _ecore_x_selection_parser_files(const char *target,
-                                                 void       *data,
-                                                 int         size,
-                                                 int         format);
-static int       _ecore_x_selection_data_files_free(void *data);
-static void *    _ecore_x_selection_parser_text(const char *target,
-                                                void       *data,
-                                                int         size,
-                                                int         format);
-static int       _ecore_x_selection_data_text_free(void *data);
-static void *    _ecore_x_selection_parser_targets(const char *target,
-                                                   void       *data,
-                                                   int         size,
-                                                   int         format);
-static int       _ecore_x_selection_data_targets_free(void *data);
+static Eina_Bool _ecore_x_selection_converter_text(char *target,
+                                                   void *data,
+                                                   int size,
+                                                   void **data_ret,
+                                                   int *size_ret,
+                                                   Ecore_X_Atom *tprop,
+                                                   int *);
+static int   _ecore_x_selection_data_default_free(void *data);
+static void *_ecore_x_selection_parser_files(const char *target,
+                                             void *data,
+                                             int size,
+                                             int format);
+static int   _ecore_x_selection_data_files_free(void *data);
+static void *_ecore_x_selection_parser_text(const char *target,
+                                            void *data,
+                                            int size,
+                                            int format);
+static int   _ecore_x_selection_data_text_free(void *data);
+static void *_ecore_x_selection_parser_targets(const char *target,
+                                               void *data,
+                                               int size,
+                                               int format);
+static int   _ecore_x_selection_data_targets_free(void *data);
 
 #define ECORE_X_SELECTION_DATA(x) ((Ecore_X_Selection_Data *)(x))
 
@@ -70,7 +70,7 @@ _ecore_x_selection_data_init(void)
                                 _ecore_x_selection_parser_files);
    ecore_x_selection_parser_add(ECORE_X_SELECTION_TARGET_TARGETS,
                                 _ecore_x_selection_parser_targets);
-} /* _ecore_x_selection_data_init */
+}
 
 void
 _ecore_x_selection_shutdown(void)
@@ -102,27 +102,27 @@ _ecore_x_selection_shutdown(void)
         free(tmp);
      }
    parsers = NULL;
-} /* _ecore_x_selection_shutdown */
+}
 
 Ecore_X_Selection_Intern *
 _ecore_x_selection_get(Ecore_X_Atom selection)
 {
    if (selection == ECORE_X_ATOM_SELECTION_PRIMARY)
-      return &selections[0];
+     return &selections[0];
    else if (selection == ECORE_X_ATOM_SELECTION_SECONDARY)
-      return &selections[1];
+     return &selections[1];
    else if (selection == ECORE_X_ATOM_SELECTION_XDND)
-      return &selections[2];
+     return &selections[2];
    else if (selection == ECORE_X_ATOM_SELECTION_CLIPBOARD)
-      return &selections[3];
+     return &selections[3];
    else
-      return NULL;
-} /* _ecore_x_selection_get */
+     return NULL;
+}
 
 Eina_Bool
-_ecore_x_selection_set(Window       w,
-                       const void  *data,
-                       int          size,
+_ecore_x_selection_set(Window w,
+                       const void *data,
+                       int size,
                        Ecore_X_Atom selection)
 {
    int in;
@@ -130,18 +130,18 @@ _ecore_x_selection_set(Window       w,
 
    XSetSelectionOwner(_ecore_x_disp, selection, w, _ecore_x_event_last_time);
    if (XGetSelectionOwner(_ecore_x_disp, selection) != w)
-      return EINA_FALSE;
+     return EINA_FALSE;
 
    if (selection == ECORE_X_ATOM_SELECTION_PRIMARY)
-      in = 0;
+     in = 0;
    else if (selection == ECORE_X_ATOM_SELECTION_SECONDARY)
-      in = 1;
+     in = 1;
    else if (selection == ECORE_X_ATOM_SELECTION_XDND)
-      in = 2;
+     in = 2;
    else if (selection == ECORE_X_ATOM_SELECTION_CLIPBOARD)
-      in = 3;
+     in = 3;
    else
-      return EINA_FALSE;
+     return EINA_FALSE;
 
    if (data)
      {
@@ -162,7 +162,7 @@ _ecore_x_selection_set(Window       w,
      }
 
    return EINA_TRUE;
-} /* _ecore_x_selection_set */
+}
 
 /**
  * Claim ownership of the PRIMARY selection and set its data.
@@ -173,11 +173,13 @@ _ecore_x_selection_set(Window       w,
  *             claimed, or 0 if unsuccessful.
  */
 EAPI Eina_Bool
-ecore_x_selection_primary_set(Ecore_X_Window w, const void *data, int size)
+ecore_x_selection_primary_set(Ecore_X_Window w,
+                              const void *data,
+                              int size)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    return _ecore_x_selection_set(w, data, size, ECORE_X_ATOM_SELECTION_PRIMARY);
-} /* ecore_x_selection_primary_set */
+}
 
 /**
  * Release ownership of the primary selection
@@ -190,7 +192,7 @@ ecore_x_selection_primary_clear(void)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    return _ecore_x_selection_set(None, NULL, 0, ECORE_X_ATOM_SELECTION_PRIMARY);
-} /* ecore_x_selection_primary_clear */
+}
 
 /**
  * Claim ownership of the SECONDARY selection and set its data.
@@ -201,14 +203,16 @@ ecore_x_selection_primary_clear(void)
  *             claimed, or 0 if unsuccessful.
  */
 EAPI Eina_Bool
-ecore_x_selection_secondary_set(Ecore_X_Window w, const void *data, int size)
+ecore_x_selection_secondary_set(Ecore_X_Window w,
+                                const void *data,
+                                int size)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    return _ecore_x_selection_set(w,
                                  data,
                                  size,
                                  ECORE_X_ATOM_SELECTION_SECONDARY);
-} /* ecore_x_selection_secondary_set */
+}
 
 /**
  * Release ownership of the secondary selection
@@ -224,7 +228,7 @@ ecore_x_selection_secondary_clear(void)
                                  NULL,
                                  0,
                                  ECORE_X_ATOM_SELECTION_SECONDARY);
-} /* ecore_x_selection_secondary_clear */
+}
 
 /**
  * Claim ownership of the XDND selection and set its data.
@@ -235,11 +239,13 @@ ecore_x_selection_secondary_clear(void)
  *             claimed, or 0 if unsuccessful.
  */
 EAPI Eina_Bool
-ecore_x_selection_xdnd_set(Ecore_X_Window w, const void *data, int size)
+ecore_x_selection_xdnd_set(Ecore_X_Window w,
+                           const void *data,
+                           int size)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    return _ecore_x_selection_set(w, data, size, ECORE_X_ATOM_SELECTION_XDND);
-} /* ecore_x_selection_xdnd_set */
+}
 
 /**
  * Release ownership of the XDND selection
@@ -252,7 +258,7 @@ ecore_x_selection_xdnd_clear(void)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    return _ecore_x_selection_set(None, NULL, 0, ECORE_X_ATOM_SELECTION_XDND);
-} /* ecore_x_selection_xdnd_clear */
+}
 
 /**
  * Claim ownership of the CLIPBOARD selection and set its data.
@@ -266,14 +272,16 @@ ecore_x_selection_xdnd_clear(void)
  * request. The buffer must be freed when done with.
  */
 EAPI Eina_Bool
-ecore_x_selection_clipboard_set(Ecore_X_Window w, const void *data, int size)
+ecore_x_selection_clipboard_set(Ecore_X_Window w,
+                                const void *data,
+                                int size)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    return _ecore_x_selection_set(w,
                                  data,
                                  size,
                                  ECORE_X_ATOM_SELECTION_CLIPBOARD);
-} /* ecore_x_selection_clipboard_set */
+}
 
 /**
  * Release ownership of the clipboard selection
@@ -289,7 +297,7 @@ ecore_x_selection_clipboard_clear(void)
                                  NULL,
                                  0,
                                  ECORE_X_ATOM_SELECTION_CLIPBOARD);
-} /* ecore_x_selection_clipboard_clear */
+}
 
 Ecore_X_Atom
 _ecore_x_selection_target_atom_get(const char *target)
@@ -297,20 +305,20 @@ _ecore_x_selection_target_atom_get(const char *target)
    Ecore_X_Atom x_target;
 
    if (!strcmp(target, ECORE_X_SELECTION_TARGET_TEXT))
-      x_target = ECORE_X_ATOM_TEXT;
+     x_target = ECORE_X_ATOM_TEXT;
    else if (!strcmp(target, ECORE_X_SELECTION_TARGET_COMPOUND_TEXT))
-      x_target = ECORE_X_ATOM_COMPOUND_TEXT;
+     x_target = ECORE_X_ATOM_COMPOUND_TEXT;
    else if (!strcmp(target, ECORE_X_SELECTION_TARGET_STRING))
-      x_target = ECORE_X_ATOM_STRING;
+     x_target = ECORE_X_ATOM_STRING;
    else if (!strcmp(target, ECORE_X_SELECTION_TARGET_UTF8_STRING))
-      x_target = ECORE_X_ATOM_UTF8_STRING;
+     x_target = ECORE_X_ATOM_UTF8_STRING;
    else if (!strcmp(target, ECORE_X_SELECTION_TARGET_FILENAME))
-      x_target = ECORE_X_ATOM_FILE_NAME;
+     x_target = ECORE_X_ATOM_FILE_NAME;
    else
-      x_target = ecore_x_atom_get(target);
+     x_target = ecore_x_atom_get(target);
 
    return x_target;
-} /* _ecore_x_selection_target_atom_get */
+}
 
 char *
 _ecore_x_selection_target_get(Ecore_X_Atom target)
@@ -318,55 +326,58 @@ _ecore_x_selection_target_get(Ecore_X_Atom target)
    /* FIXME: Should not return mem allocated with strdup or X mixed,
     * one should use free to free, the other XFree */
    if (target == ECORE_X_ATOM_FILE_NAME)
-      return strdup(ECORE_X_SELECTION_TARGET_FILENAME);
+     return strdup(ECORE_X_SELECTION_TARGET_FILENAME);
    else if (target == ECORE_X_ATOM_STRING)
-      return strdup(ECORE_X_SELECTION_TARGET_STRING);
+     return strdup(ECORE_X_SELECTION_TARGET_STRING);
    else if (target == ECORE_X_ATOM_UTF8_STRING)
-      return strdup(ECORE_X_SELECTION_TARGET_UTF8_STRING);
+     return strdup(ECORE_X_SELECTION_TARGET_UTF8_STRING);
    else if (target == ECORE_X_ATOM_TEXT)
-      return strdup(ECORE_X_SELECTION_TARGET_TEXT);
+     return strdup(ECORE_X_SELECTION_TARGET_TEXT);
    else
-      return XGetAtomName(_ecore_x_disp, target);
-} /* _ecore_x_selection_target_get */
+     return XGetAtomName(_ecore_x_disp, target);
+}
 
 static void
 _ecore_x_selection_request(Ecore_X_Window w,
-                           Ecore_X_Atom   selection,
-                           const char    *target_str)
+                           Ecore_X_Atom selection,
+                           const char *target_str)
 {
    Ecore_X_Atom target, prop;
 
    target = _ecore_x_selection_target_atom_get(target_str);
 
    if (selection == ECORE_X_ATOM_SELECTION_PRIMARY)
-      prop = ECORE_X_ATOM_SELECTION_PROP_PRIMARY;
+     prop = ECORE_X_ATOM_SELECTION_PROP_PRIMARY;
    else if (selection == ECORE_X_ATOM_SELECTION_SECONDARY)
-      prop = ECORE_X_ATOM_SELECTION_PROP_SECONDARY;
+     prop = ECORE_X_ATOM_SELECTION_PROP_SECONDARY;
    else if (selection == ECORE_X_ATOM_SELECTION_CLIPBOARD)
-      prop = ECORE_X_ATOM_SELECTION_PROP_CLIPBOARD;
+     prop = ECORE_X_ATOM_SELECTION_PROP_CLIPBOARD;
    else
-      return;
+     return;
 
    XConvertSelection(_ecore_x_disp, selection, target, prop,
                      w, CurrentTime);
-} /* _ecore_x_selection_request */
+}
 
 EAPI void
-ecore_x_selection_primary_request(Ecore_X_Window w, const char *target)
+ecore_x_selection_primary_request(Ecore_X_Window w,
+                                  const char *target)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    _ecore_x_selection_request(w, ECORE_X_ATOM_SELECTION_PRIMARY, target);
-} /* ecore_x_selection_primary_request */
+}
 
 EAPI void
-ecore_x_selection_secondary_request(Ecore_X_Window w, const char *target)
+ecore_x_selection_secondary_request(Ecore_X_Window w,
+                                    const char *target)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    _ecore_x_selection_request(w, ECORE_X_ATOM_SELECTION_SECONDARY, target);
-} /* ecore_x_selection_secondary_request */
+}
 
 EAPI void
-ecore_x_selection_xdnd_request(Ecore_X_Window w, const char *target)
+ecore_x_selection_xdnd_request(Ecore_X_Window w,
+                               const char *target)
 {
    Ecore_X_Atom atom;
    Ecore_X_DND_Target *_target;
@@ -377,24 +388,25 @@ ecore_x_selection_xdnd_request(Ecore_X_Window w, const char *target)
    XConvertSelection(_ecore_x_disp, ECORE_X_ATOM_SELECTION_XDND, atom,
                      ECORE_X_ATOM_SELECTION_PROP_XDND, w,
                      _target->time);
-} /* ecore_x_selection_xdnd_request */
+}
 
 EAPI void
-ecore_x_selection_clipboard_request(Ecore_X_Window w, const char *target)
+ecore_x_selection_clipboard_request(Ecore_X_Window w,
+                                    const char *target)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    _ecore_x_selection_request(w, ECORE_X_ATOM_SELECTION_CLIPBOARD, target);
-} /* ecore_x_selection_clipboard_request */
+}
 
 EAPI void
 ecore_x_selection_converter_atom_add(Ecore_X_Atom target,
-                                     Eina_Bool  (*func)(char         *target,
-                                                        void         *data,
-                                                        int           size,
-                                                        void        **data_ret,
-                                                        int          *size_ret,
-                                                        Ecore_X_Atom *ttype,
-                                                        int          *tsize))
+                                     Eina_Bool (*func)(char *target,
+                                                       void *data,
+                                                       int size,
+                                                       void **data_ret,
+                                                       int *size_ret,
+                                                       Ecore_X_Atom *ttype,
+                                                       int *tsize))
 {
    Ecore_X_Selection_Converter *cnv;
 
@@ -411,9 +423,9 @@ ecore_x_selection_converter_atom_add(Ecore_X_Atom target,
                }
 
              if (cnv->next)
-                cnv = cnv->next;
+               cnv = cnv->next;
              else
-                break;
+               break;
           }
 
         cnv->next = calloc(1, sizeof(Ecore_X_Selection_Converter));
@@ -429,28 +441,28 @@ ecore_x_selection_converter_atom_add(Ecore_X_Atom target,
 
    cnv->target = target;
    cnv->convert = func;
-} /* ecore_x_selection_converter_atom_add */
+}
 
 EAPI void
-ecore_x_selection_converter_add(char                             *target,
-                                Eina_Bool                       (*func)(char *target,
-                                                           void  *data,
-                                                           int    size,
-                                                           void **data_ret,
-                                                           int   *size_ret,
-                                                           Ecore_X_Atom *,
-                                                           int *))
+ecore_x_selection_converter_add(char *target,
+                                Eina_Bool (*func)(char *target,
+                                                  void *data,
+                                                  int size,
+                                                  void **data_ret,
+                                                  int *size_ret,
+                                                  Ecore_X_Atom *,
+                                                  int *))
 {
    Ecore_X_Atom x_target;
 
    if (!func || !target)
-      return;
+     return;
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    x_target = _ecore_x_selection_target_atom_get(target);
 
    ecore_x_selection_converter_atom_add(x_target, func);
-} /* ecore_x_selection_converter_add */
+}
 
 EAPI void
 ecore_x_selection_converter_atom_del(Ecore_X_Atom target)
@@ -466,7 +478,7 @@ ecore_x_selection_converter_atom_del(Ecore_X_Atom target)
         if (cnv->target == target)
           {
              if (prev_cnv)
-                prev_cnv->next = cnv->next;
+               prev_cnv->next = cnv->next;
              else
                {
                   converters = cnv->next; /* This was the first converter */
@@ -480,7 +492,7 @@ ecore_x_selection_converter_atom_del(Ecore_X_Atom target)
         prev_cnv = cnv;
         cnv = cnv->next;
      }
-} /* ecore_x_selection_converter_atom_del */
+}
 
 EAPI void
 ecore_x_selection_converter_del(char *target)
@@ -488,19 +500,19 @@ ecore_x_selection_converter_del(char *target)
    Ecore_X_Atom x_target;
 
    if (!target)
-      return;
+     return;
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    x_target = _ecore_x_selection_target_atom_get(target);
    ecore_x_selection_converter_atom_del(x_target);
-} /* ecore_x_selection_converter_del */
+}
 
 EAPI Eina_Bool
 ecore_x_selection_notify_send(Ecore_X_Window requestor,
-                              Ecore_X_Atom   selection,
-                              Ecore_X_Atom   target,
-                              Ecore_X_Atom   property,
-                              Ecore_X_Time   time)
+                              Ecore_X_Atom selection,
+                              Ecore_X_Atom target,
+                              Ecore_X_Atom property,
+                              Ecore_X_Time tim)
 {
    XEvent xev;
    XSelectionEvent xnotify;
@@ -512,22 +524,22 @@ ecore_x_selection_notify_send(Ecore_X_Window requestor,
    xnotify.selection = selection;
    xnotify.target = target;
    xnotify.property = property;
-   xnotify.time = time;
+   xnotify.time = tim;
    xnotify.send_event = True;
    xnotify.serial = 0;
 
    xev.xselection = xnotify;
-   return ((XSendEvent(_ecore_x_disp, requestor, False, 0, &xev) > 0) ? EINA_TRUE : EINA_FALSE);
-} /* ecore_x_selection_notify_send */
+   return (XSendEvent(_ecore_x_disp, requestor, False, 0, &xev) > 0) ? EINA_TRUE : EINA_FALSE;
+}
 
 /* Locate and run conversion callback for specified selection target */
 EAPI Eina_Bool
-ecore_x_selection_convert(Ecore_X_Atom  selection,
-                          Ecore_X_Atom  target,
-                          void        **data_ret,
-                          int          *size,
+ecore_x_selection_convert(Ecore_X_Atom selection,
+                          Ecore_X_Atom target,
+                          void **data_ret,
+                          int *size,
                           Ecore_X_Atom *targtype,
-                          int          *typesize)
+                          int *typesize)
 {
    Ecore_X_Selection_Intern *sel;
    Ecore_X_Selection_Converter *cnv;
@@ -552,7 +564,7 @@ ecore_x_selection_convert(Ecore_X_Atom  selection,
                   return r;
                }
              else
-                return EINA_FALSE;
+               return EINA_FALSE;
           }
      }
 
@@ -560,48 +572,50 @@ ecore_x_selection_convert(Ecore_X_Atom  selection,
    return EINA_FALSE;
 
    /* Default, just return the data
-      *data_ret = malloc(sel->length);
+    * data_ret = malloc(sel->length);
       memcpy(*data_ret, sel->data, sel->length);
       free(tgt_str);
       return 1;
     */
-} /* ecore_x_selection_convert */
+}
 
 /* TODO: We need to work out a mechanism for automatic conversion to any requested
  * locale using Ecore_Txt functions */
 /* Converter for standard non-utf8 text targets */
 static Eina_Bool
-_ecore_x_selection_converter_text(char         *target,
-                                  void         *data,
-                                  int           size,
-                                  void        **data_ret,
-                                  int          *size_ret,
+_ecore_x_selection_converter_text(char *target,
+                                  void *data,
+                                  int size,
+                                  void **data_ret,
+                                  int *size_ret,
                                   Ecore_X_Atom *targprop __UNUSED__,
-                                  int          *s __UNUSED__)
+                                  int *s __UNUSED__)
 {
    XTextProperty text_prop;
    char *mystr;
    XICCEncodingStyle style;
 
    if (!data || !size)
-      return EINA_FALSE;
+     return EINA_FALSE;
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (!strcmp(target, ECORE_X_SELECTION_TARGET_TEXT))
-      style = XTextStyle;
+     style = XTextStyle;
    else if (!strcmp(target, ECORE_X_SELECTION_TARGET_COMPOUND_TEXT))
-      style = XCompoundTextStyle;
+     style = XCompoundTextStyle;
    else if (!strcmp(target, ECORE_X_SELECTION_TARGET_STRING))
-      style = XStringStyle;
+     style = XStringStyle;
 
 #ifdef X_HAVE_UTF8_STRING
    else if (!strcmp(target, ECORE_X_SELECTION_TARGET_UTF8_STRING))
-      style = XUTF8StringStyle;
+     style = XUTF8StringStyle;
 #endif /* ifdef X_HAVE_UTF8_STRING */
    else
-      return EINA_FALSE;
-   if (!(mystr = strdup(data)))
-      return EINA_FALSE;
+     return EINA_FALSE;
+
+   mystr = alloca(size + 1);
+   memcpy(mystr, data, size);
+   mystr[size] = '\0';
 
 #ifdef X_HAVE_UTF8_STRING
    if (Xutf8TextListToTextProperty(_ecore_x_disp, &mystr, 1, style,
@@ -611,13 +625,11 @@ _ecore_x_selection_converter_text(char         *target,
         *data_ret = malloc(bufsize);
         if (!*data_ret)
           {
-             free(mystr);
              return EINA_FALSE;
           }
         memcpy(*data_ret, text_prop.value, bufsize);
         *size_ret = bufsize;
         XFree(text_prop.value);
-        free(mystr);
         return EINA_TRUE;
      }
 
@@ -631,17 +643,15 @@ _ecore_x_selection_converter_text(char         *target,
         memcpy(*data_ret, text_prop.value, bufsize);
         *size_ret = bufsize;
         XFree(text_prop.value);
-        free(mystr);
         return EINA_TRUE;
      }
 
 #endif /* ifdef X_HAVE_UTF8_STRING */
    else
      {
-        free(mystr);
         return EINA_TRUE;
      }
-} /* _ecore_x_selection_converter_text */
+}
 
 EAPI void
 ecore_x_selection_parser_add(const char *target,
@@ -652,7 +662,7 @@ ecore_x_selection_parser_add(const char *target,
    Ecore_X_Selection_Parser *prs;
 
    if (!target)
-      return;
+     return;
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    prs = parsers;
@@ -682,7 +692,7 @@ ecore_x_selection_parser_add(const char *target,
 
    prs->target = strdup(target);
    prs->parse = func;
-} /* ecore_x_selection_parser_add */
+}
 
 EAPI void
 ecore_x_selection_parser_del(const char *target)
@@ -690,7 +700,7 @@ ecore_x_selection_parser_del(const char *target)
    Ecore_X_Selection_Parser *prs, *prev_prs;
 
    if (!target)
-      return;
+     return;
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    prev_prs = NULL;
@@ -701,7 +711,7 @@ ecore_x_selection_parser_del(const char *target)
         if (!strcmp(prs->target, target))
           {
              if (prev_prs)
-                prev_prs->next = prs->next;
+               prev_prs->next = prs->next;
              else
                {
                   parsers = prs->next; /* This was the first parser */
@@ -716,11 +726,43 @@ ecore_x_selection_parser_del(const char *target)
         prev_prs = prs;
         prs = prs->next;
      }
-} /* ecore_x_selection_parser_del */
+}
+
+/**
+ * Change the owner and last-change time for the specified selection.
+ * @param win The owner of the specified atom.
+ * @param atom The selection atom
+ * @param tim Specifies the time
+ * @since 1.1.0
+ */
+EAPI void
+ecore_x_selection_owner_set(Ecore_X_Window win,
+                            Ecore_X_Atom atom,
+                            Ecore_X_Time tim)
+{
+   XSetSelectionOwner(_ecore_x_disp, atom, win, tim);
+}
+
+/**
+ * Return the window that currently owns the specified selection.
+ *
+ * @param atom The specified selection atom.
+ *
+ * @return The window that currently owns the specified selection.
+ * @since 1.1.0
+ */
+EAPI Ecore_X_Window
+ecore_x_selection_owner_get(Ecore_X_Atom atom)
+{
+   return XGetSelectionOwner(_ecore_x_disp, atom);
+}
 
 /* Locate and run conversion callback for specified selection target */
 void *
-_ecore_x_selection_parse(const char *target, void *data, int size, int format)
+_ecore_x_selection_parse(const char *target,
+                         void *data,
+                         int size,
+                         int format)
 {
    Ecore_X_Selection_Parser *prs;
    Ecore_X_Selection_Data *sel;
@@ -730,7 +772,7 @@ _ecore_x_selection_parse(const char *target, void *data, int size, int format)
         if (!strcmp(prs->target, target))
           {
              sel = prs->parse(target, data, size, format);
-	     if (sel) return sel;
+             if (sel) return sel;
           }
      }
 
@@ -742,7 +784,7 @@ _ecore_x_selection_parse(const char *target, void *data, int size, int format)
    sel->format = format;
    sel->data = data;
    return sel;
-} /* _ecore_x_selection_parse */
+}
 
 static int
 _ecore_x_selection_data_default_free(void *data)
@@ -753,13 +795,13 @@ _ecore_x_selection_data_default_free(void *data)
    free(sel->data);
    free(sel);
    return 1;
-} /* _ecore_x_selection_data_default_free */
+}
 
 static void *
 _ecore_x_selection_parser_files(const char *target,
-                                void       *_data,
-                                int         size,
-                                int format  __UNUSED__)
+                                void *_data,
+                                int size,
+                                int format __UNUSED__)
 {
    Ecore_X_Selection_Data_Files *sel;
    char *t, *data = _data;
@@ -769,7 +811,7 @@ _ecore_x_selection_parser_files(const char *target,
 
    if (strcmp(target, "text/uri-list") &&
        strcmp(target, "_NETSCAPE_URL"))
-      return NULL;
+     return NULL;
 
    sel = calloc(1, sizeof(Ecore_X_Selection_Data_Files));
    if (!sel) return NULL;
@@ -800,15 +842,16 @@ _ecore_x_selection_parser_files(const char *target,
    while ((is < size) && (data[is]))
      {
         if ((i == 0) && (data[is] == '#'))
-           for (; ((data[is]) && (data[is] != '\n')); is++) ;
+          for (; ((data[is]) && (data[is] != '\n')); is++) ;
         else
           {
              if ((data[is] != '\r') &&
                  (data[is] != '\n'))
-                tmp[i++] = data[is++];
+               tmp[i++] = data[is++];
              else
                {
-                  while ((data[is] == '\r') || (data[is] == '\n')) is++;
+                  while ((data[is] == '\r') || (data[is] == '\n'))
+                    is++;
                   tmp[i] = 0;
                   sel->num_files++;
                   t2 = realloc(sel->files, sel->num_files * sizeof(char *));
@@ -841,7 +884,7 @@ _ecore_x_selection_parser_files(const char *target,
    ECORE_X_SELECTION_DATA(sel)->length = sel->num_files;
 
    return ECORE_X_SELECTION_DATA(sel);
-} /* _ecore_x_selection_parser_files */
+}
 
 static int
 _ecore_x_selection_data_files_free(void *data)
@@ -853,19 +896,19 @@ _ecore_x_selection_data_files_free(void *data)
    if (sel->files)
      {
         for (i = 0; i < sel->num_files; i++)
-           free(sel->files[i]);
+          free(sel->files[i]);
         free(sel->files);
      }
 
    free(sel);
    return 0;
-} /* _ecore_x_selection_data_files_free */
+}
 
 static void *
 _ecore_x_selection_parser_text(const char *target __UNUSED__,
-                               void              *_data,
-                               int                size,
-                               int format         __UNUSED__)
+                               void *_data,
+                               int size,
+                               int format __UNUSED__)
 {
    Ecore_X_Selection_Data_Text *sel;
    unsigned char *data = _data;
@@ -893,7 +936,7 @@ _ecore_x_selection_parser_text(const char *target __UNUSED__,
    ECORE_X_SELECTION_DATA(sel)->data = data;
    ECORE_X_SELECTION_DATA(sel)->free = _ecore_x_selection_data_text_free;
    return sel;
-} /* _ecore_x_selection_parser_text */
+}
 
 static int
 _ecore_x_selection_data_text_free(void *data)
@@ -904,13 +947,13 @@ _ecore_x_selection_data_text_free(void *data)
    free(sel->text);
    free(sel);
    return 1;
-} /* _ecore_x_selection_data_text_free */
+}
 
 static void *
 _ecore_x_selection_parser_targets(const char *target __UNUSED__,
-                                  void              *data,
-                                  int                size,
-                                  int format         __UNUSED__)
+                                  void *data,
+                                  int size,
+                                  int format __UNUSED__)
 {
    Ecore_X_Selection_Data_Targets *sel;
    unsigned long *targets;
@@ -928,14 +971,14 @@ _ecore_x_selection_parser_targets(const char *target __UNUSED__,
         return NULL;
      }
    for (i = 2; i < size; i++)
-      sel->targets[i - 2] = XGetAtomName(_ecore_x_disp, targets[i]);
+     sel->targets[i - 2] = XGetAtomName(_ecore_x_disp, targets[i]);
 
    ECORE_X_SELECTION_DATA(sel)->free = _ecore_x_selection_data_targets_free;
    ECORE_X_SELECTION_DATA(sel)->content = ECORE_X_SELECTION_CONTENT_TARGETS;
    ECORE_X_SELECTION_DATA(sel)->length = size;
    ECORE_X_SELECTION_DATA(sel)->data = data;
    return sel;
-} /* _ecore_x_selection_parser_targets */
+}
 
 static int
 _ecore_x_selection_data_targets_free(void *data)
@@ -948,12 +991,12 @@ _ecore_x_selection_data_targets_free(void *data)
    if (sel->targets)
      {
         for (i = 0; i < sel->num_targets; i++)
-           XFree(sel->targets[i]);
+          XFree(sel->targets[i]);
         free(sel->targets);
      }
 
    free(ECORE_X_SELECTION_DATA(sel)->data);
    free(sel);
    return 1;
-} /* _ecore_x_selection_data_targets_free */
+}
 
