@@ -13,7 +13,11 @@
 #include <string.h>
 
 EAPI Eina_Bool
+#ifdef ECORE_XTEST
 ecore_x_test_fake_key_down(const char *key)
+#else
+ecore_x_test_fake_key_down(const char *key __UNUSED__)
+#endif
 {
 #ifdef ECORE_XTEST
    KeyCode keycode = 0;
@@ -21,27 +25,31 @@ ecore_x_test_fake_key_down(const char *key)
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (!strncmp(key, "Keycode-", 8))
-      keycode = atoi(key + 8);
+     keycode = atoi(key + 8);
    else
      {
         keysym = XStringToKeysym(key);
         if (keysym == NoSymbol)
-           return EINA_FALSE;
+          return EINA_FALSE;
 
         keycode = XKeysymToKeycode(_ecore_x_disp, keysym);
      }
 
    if (keycode == 0)
-      return EINA_FALSE;
+     return EINA_FALSE;
 
    return XTestFakeKeyEvent(_ecore_x_disp, keycode, 1, 0) ? EINA_TRUE : EINA_FALSE;
 #else /* ifdef ECORE_XTEST */
    return EINA_FALSE;
 #endif /* ifdef ECORE_XTEST */
-} /* ecore_x_test_fake_key_down */
+}
 
 EAPI Eina_Bool
+#ifdef ECORE_XTEST
 ecore_x_test_fake_key_up(const char *key)
+#else
+ecore_x_test_fake_key_up(const char *key __UNUSED__)
+#endif
 {
 #ifdef ECORE_XTEST
    KeyCode keycode = 0;
@@ -49,27 +57,31 @@ ecore_x_test_fake_key_up(const char *key)
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (!strncmp(key, "Keycode-", 8))
-      keycode = atoi(key + 8);
+     keycode = atoi(key + 8);
    else
      {
         keysym = XStringToKeysym(key);
         if (keysym == NoSymbol)
-           return EINA_FALSE;
+          return EINA_FALSE;
 
         keycode = XKeysymToKeycode(_ecore_x_disp, keysym);
      }
 
    if (keycode == 0)
-      return EINA_FALSE;
+     return EINA_FALSE;
 
    return XTestFakeKeyEvent(_ecore_x_disp, keycode, 0, 0) ? EINA_TRUE : EINA_FALSE;
 #else /* ifdef ECORE_XTEST */
    return EINA_FALSE;
 #endif /* ifdef ECORE_XTEST */
-} /* ecore_x_test_fake_key_up */
+}
 
 EAPI Eina_Bool
+#ifdef ECORE_XTEST
 ecore_x_test_fake_key_press(const char *key)
+#else
+ecore_x_test_fake_key_press(const char *key __UNUSED__)
+#endif
 {
 #ifdef ECORE_XTEST
    KeyCode keycode = 0;
@@ -78,23 +90,23 @@ ecore_x_test_fake_key_press(const char *key)
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (!strncmp(key, "Keycode-", 8))
-      keycode = atoi(key + 8);
+     keycode = atoi(key + 8);
    else
      {
         keysym = XStringToKeysym(key);
         if (keysym == NoSymbol)
-           return EINA_FALSE;
+          return EINA_FALSE;
 
         keycode = XKeysymToKeycode(_ecore_x_disp, keysym);
         if (XKeycodeToKeysym(_ecore_x_disp, keycode, 0) != keysym)
           {
              if (XKeycodeToKeysym(_ecore_x_disp, keycode, 1) == keysym)
-                shift = 1;
+               shift = 1;
              else
-                keycode = 0;
+               keycode = 0;
           }
         else
-           shift = 0;
+          shift = 0;
      }
 
    if (keycode == 0)
@@ -120,24 +132,36 @@ ecore_x_test_fake_key_press(const char *key)
      }
 
    if (shift)
-      XTestFakeKeyEvent(_ecore_x_disp,
-                        XKeysymToKeycode(_ecore_x_disp, XK_Shift_L), 1, 0);
+     XTestFakeKeyEvent(_ecore_x_disp,
+                       XKeysymToKeycode(_ecore_x_disp, XK_Shift_L), 1, 0);
 
    XTestFakeKeyEvent(_ecore_x_disp, keycode, 1, 0);
    XTestFakeKeyEvent(_ecore_x_disp, keycode, 0, 0);
    if (shift)
-      XTestFakeKeyEvent(_ecore_x_disp,
-                        XKeysymToKeycode(_ecore_x_disp, XK_Shift_L), 0, 0);
+     XTestFakeKeyEvent(_ecore_x_disp,
+                       XKeysymToKeycode(_ecore_x_disp, XK_Shift_L), 0, 0);
 
    return EINA_TRUE;
 #else /* ifdef ECORE_XTEST */
    return EINA_FALSE;
 #endif /* ifdef ECORE_XTEST */
-} /* ecore_x_test_fake_key_press */
+}
 
 EAPI const char *
 ecore_x_keysym_string_get(int keysym)
 {
    return XKeysymToString(keysym);
-} /* ecore_x_keysym_string_get */
+}
 
+EAPI int 
+ecore_x_keysym_keycode_get(const char *keyname)
+{
+   int keycode = 0;
+
+   if (!strncmp(keyname, "Keycode-", 8))
+     keycode = atoi(keyname + 8);
+   else
+     keycode = XKeysymToKeycode(_ecore_x_disp, XStringToKeysym(keyname));
+
+   return keycode;
+}
