@@ -12,16 +12,18 @@
 #include <inttypes.h>
 #include <limits.h>
 
-#define _ATOM_SET_CARD32(win, atom, p_val, cnt)\
-   XChangeProperty(_ecore_x_disp, win, atom, XA_CARDINAL, 32, PropModeReplace,\
-                   (unsigned char *)p_val, cnt)
+#define _ATOM_SET_CARD32(win, atom, p_val, cnt)                               \
+  XChangeProperty(_ecore_x_disp, win, atom, XA_CARDINAL, 32, PropModeReplace, \
+                  (unsigned char *)p_val, cnt)
 
 /*
  * Set CARD32 (array) property
  */
 EAPI void
-ecore_x_window_prop_card32_set(Ecore_X_Window win, Ecore_X_Atom atom,
-                               unsigned int *val, unsigned int num)
+ecore_x_window_prop_card32_set(Ecore_X_Window win,
+                               Ecore_X_Atom atom,
+                               unsigned int *val,
+                               unsigned int num)
 {
 #if SIZEOF_INT == SIZEOF_LONG
    _ATOM_SET_CARD32(win, atom, val, num);
@@ -32,14 +34,14 @@ ecore_x_window_prop_card32_set(Ecore_X_Window win, Ecore_X_Atom atom,
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    v2 = malloc(num * sizeof(long));
    if (!v2)
-      return;
+     return;
 
    for (i = 0; i < num; i++)
-      v2[i] = val[i];
+     v2[i] = val[i];
    _ATOM_SET_CARD32(win, atom, v2, num);
    free(v2);
 #endif /* if SIZEOF_INT == SIZEOF_LONG */
-} /* ecore_x_window_prop_card32_set */
+}
 
 /*
  * Get CARD32 (array) property
@@ -50,8 +52,10 @@ ecore_x_window_prop_card32_set(Ecore_X_Window win, Ecore_X_Atom atom,
  * Note: Return value 0 means that the property exists but has no elements.
  */
 EAPI int
-ecore_x_window_prop_card32_get(Ecore_X_Window win, Ecore_X_Atom atom,
-                               unsigned int *val, unsigned int len)
+ecore_x_window_prop_card32_get(Ecore_X_Window win,
+                               Ecore_X_Atom atom,
+                               unsigned int *val,
+                               unsigned int len)
 {
    unsigned char *prop_ret;
    Atom type_ret;
@@ -65,27 +69,27 @@ ecore_x_window_prop_card32_get(Ecore_X_Window win, Ecore_X_Atom atom,
    if (XGetWindowProperty(_ecore_x_disp, win, atom, 0, 0x7fffffff, False,
                           XA_CARDINAL, &type_ret, &format_ret, &num_ret,
                           &bytes_after, &prop_ret) != Success)
-      return -1;
+     return -1;
 
    if (type_ret != XA_CARDINAL || format_ret != 32)
-      num = -1;
+     num = -1;
    else if (num_ret == 0 || !prop_ret)
-      num = 0;
+     num = 0;
    else
      {
         if (num_ret < len)
-           len = num_ret;
+          len = num_ret;
 
         for (i = 0; i < len; i++)
-           val[i] = ((unsigned long *)prop_ret)[i];
+          val[i] = ((unsigned long *)prop_ret)[i];
         num = len;
      }
 
    if (prop_ret)
-      XFree(prop_ret);
+     XFree(prop_ret);
 
    return num;
-} /* ecore_x_window_prop_card32_get */
+}
 
 /*
  * Get CARD32 (array) property of any length
@@ -95,7 +99,8 @@ ecore_x_window_prop_card32_get(Ecore_X_Window win, Ecore_X_Atom atom,
  * Note: Return value 0 means that the property exists but has no elements.
  */
 EAPI int
-ecore_x_window_prop_card32_list_get(Ecore_X_Window win, Ecore_X_Atom atom,
+ecore_x_window_prop_card32_list_get(Ecore_X_Window win,
+                                    Ecore_X_Atom atom,
                                     unsigned int **plst)
 {
    unsigned char *prop_ret;
@@ -111,33 +116,40 @@ ecore_x_window_prop_card32_list_get(Ecore_X_Window win, Ecore_X_Atom atom,
    if (XGetWindowProperty(_ecore_x_disp, win, atom, 0, 0x7fffffff, False,
                           XA_CARDINAL, &type_ret, &format_ret, &num_ret,
                           &bytes_after, &prop_ret) != Success)
-      return -1;
+     return -1;
 
-   if (type_ret != XA_CARDINAL || format_ret != 32)
-      num = -1;
-   else if (num_ret == 0 || !prop_ret)
-      num = 0;
+   if ((type_ret != XA_CARDINAL) || (format_ret != 32))
+     num = -1;
+   else if ((num_ret == 0) || (!prop_ret))
+     num = 0;
    else
      {
         val = malloc(num_ret * sizeof(unsigned int));
+        if (!val)
+          {
+             if (prop_ret) XFree(prop_ret);
+             return -1;
+          }
         for (i = 0; i < num_ret; i++)
-           val[i] = ((unsigned long *)prop_ret)[i];
+          val[i] = ((unsigned long *)prop_ret)[i];
         num = num_ret;
         *plst = val;
      }
 
    if (prop_ret)
-      XFree(prop_ret);
+     XFree(prop_ret);
 
    return num;
-} /* ecore_x_window_prop_card32_list_get */
+}
 
 /*
  * Set X ID (array) property
  */
 EAPI void
-ecore_x_window_prop_xid_set(Ecore_X_Window win, Ecore_X_Atom atom,
-                            Ecore_X_Atom type, Ecore_X_ID *lst,
+ecore_x_window_prop_xid_set(Ecore_X_Window win,
+                            Ecore_X_Atom atom,
+                            Ecore_X_Atom type,
+                            Ecore_X_ID *lst,
                             unsigned int num)
 {
 #if SIZEOF_INT == SIZEOF_LONG
@@ -150,15 +162,15 @@ ecore_x_window_prop_xid_set(Ecore_X_Window win, Ecore_X_Atom atom,
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    pl = malloc(num * sizeof(long));
    if (!pl)
-      return;
+     return;
 
    for (i = 0; i < num; i++)
-      pl[i] = lst[i];
+     pl[i] = lst[i];
    XChangeProperty(_ecore_x_disp, win, atom, type, 32, PropModeReplace,
                    (unsigned char *)pl, num);
    free(pl);
 #endif /* if SIZEOF_INT == SIZEOF_LONG */
-} /* ecore_x_window_prop_xid_set */
+}
 
 /*
  * Get X ID (array) property
@@ -169,8 +181,10 @@ ecore_x_window_prop_xid_set(Ecore_X_Window win, Ecore_X_Atom atom,
  * Note: Return value 0 means that the property exists but has no elements.
  */
 EAPI int
-ecore_x_window_prop_xid_get(Ecore_X_Window win, Ecore_X_Atom atom,
-                            Ecore_X_Atom type, Ecore_X_ID *lst,
+ecore_x_window_prop_xid_get(Ecore_X_Window win,
+                            Ecore_X_Atom atom,
+                            Ecore_X_Atom type,
+                            Ecore_X_ID *lst,
                             unsigned int len)
 {
    unsigned char *prop_ret;
@@ -185,27 +199,27 @@ ecore_x_window_prop_xid_get(Ecore_X_Window win, Ecore_X_Atom atom,
    if (XGetWindowProperty(_ecore_x_disp, win, atom, 0, 0x7fffffff, False,
                           type, &type_ret, &format_ret, &num_ret,
                           &bytes_after, &prop_ret) != Success)
-      return -1;
+     return -1;
 
    if (type_ret != type || format_ret != 32)
-      num = -1;
+     num = -1;
    else if (num_ret == 0 || !prop_ret)
-      num = 0;
+     num = 0;
    else
      {
         if (num_ret < len)
-           len = num_ret;
+          len = num_ret;
 
         for (i = 0; i < len; i++)
-           lst[i] = ((unsigned long *)prop_ret)[i];
+          lst[i] = ((unsigned long *)prop_ret)[i];
         num = len;
      }
 
    if (prop_ret)
-      XFree(prop_ret);
+     XFree(prop_ret);
 
    return num;
-} /* ecore_x_window_prop_xid_get */
+}
 
 /*
  * Get X ID (array) property
@@ -216,8 +230,10 @@ ecore_x_window_prop_xid_get(Ecore_X_Window win, Ecore_X_Atom atom,
  * Note: Return value 0 means that the property exists but has no elements.
  */
 EAPI int
-ecore_x_window_prop_xid_list_get(Ecore_X_Window win, Ecore_X_Atom atom,
-                                 Ecore_X_Atom type, Ecore_X_ID **val)
+ecore_x_window_prop_xid_list_get(Ecore_X_Window win,
+                                 Ecore_X_Atom atom,
+                                 Ecore_X_Atom type,
+                                 Ecore_X_ID **val)
 {
    unsigned char *prop_ret;
    Atom type_ret;
@@ -233,33 +249,36 @@ ecore_x_window_prop_xid_list_get(Ecore_X_Window win, Ecore_X_Atom atom,
    if (XGetWindowProperty(_ecore_x_disp, win, atom, 0, 0x7fffffff, False,
                           type, &type_ret, &format_ret, &num_ret,
                           &bytes_after, &prop_ret) != Success)
-      return -1;
+     return -1;
 
    if (type_ret != type || format_ret != 32)
-      num = -1;
+     num = -1;
    else if (num_ret == 0 || !prop_ret)
-      num = 0;
+     num = 0;
    else
      {
         alst = malloc(num_ret * sizeof(Ecore_X_ID));
         for (i = 0; i < num_ret; i++)
-           alst[i] = ((unsigned long *)prop_ret)[i];
+          alst[i] = ((unsigned long *)prop_ret)[i];
         num = num_ret;
         *val = alst;
      }
 
    if (prop_ret)
-      XFree(prop_ret);
+     XFree(prop_ret);
 
    return num;
-} /* ecore_x_window_prop_xid_list_get */
+}
 
 /*
  * Remove/add/toggle X ID list item.
  */
 EAPI void
-ecore_x_window_prop_xid_list_change(Ecore_X_Window win, Ecore_X_Atom atom,
-                                    Ecore_X_Atom type, Ecore_X_ID item, int op)
+ecore_x_window_prop_xid_list_change(Ecore_X_Window win,
+                                    Ecore_X_Atom atom,
+                                    Ecore_X_Atom type,
+                                    Ecore_X_ID item,
+                                    int op)
 {
    Ecore_X_ID *lst;
    int i, num;
@@ -275,24 +294,24 @@ ecore_x_window_prop_xid_list_change(Ecore_X_Window win, Ecore_X_Atom atom,
    for (i = 0; i < num; i++)
      {
         if (lst[i] == item)
-           break;
+          break;
      }
 
    if (i < num)
      {
         /* Was in list */
         if (op == ECORE_X_PROP_LIST_ADD)
-           goto done;  /* Remove it */
+          goto done;  /* Remove it */
 
         num--;
         for (; i < num; i++)
-           lst[i] = lst[i + 1];
+          lst[i] = lst[i + 1];
      }
    else
      {
         /* Was not in list */
         if (op == ECORE_X_PROP_LIST_REMOVE)
-           goto done;  /* Add it */
+          goto done;  /* Add it */
 
         num++;
         lst = realloc(lst, num * sizeof(Ecore_X_ID));
@@ -303,19 +322,21 @@ ecore_x_window_prop_xid_list_change(Ecore_X_Window win, Ecore_X_Atom atom,
 
 done:
    if (lst)
-      free(lst);
-} /* ecore_x_window_prop_xid_list_change */
+     free(lst);
+}
 
 /*
  * Set Atom (array) property
  */
 EAPI void
-ecore_x_window_prop_atom_set(Ecore_X_Window win, Ecore_X_Atom atom,
-                             Ecore_X_Atom *lst, unsigned int num)
+ecore_x_window_prop_atom_set(Ecore_X_Window win,
+                             Ecore_X_Atom atom,
+                             Ecore_X_Atom *lst,
+                             unsigned int num)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    ecore_x_window_prop_xid_set(win, atom, XA_ATOM, lst, num);
-} /* ecore_x_window_prop_atom_set */
+}
 
 /*
  * Get Atom (array) property
@@ -326,12 +347,14 @@ ecore_x_window_prop_atom_set(Ecore_X_Window win, Ecore_X_Atom atom,
  * Note: Return value 0 means that the property exists but has no elements.
  */
 EAPI int
-ecore_x_window_prop_atom_get(Ecore_X_Window win, Ecore_X_Atom atom,
-                             Ecore_X_Atom *lst, unsigned int len)
+ecore_x_window_prop_atom_get(Ecore_X_Window win,
+                             Ecore_X_Atom atom,
+                             Ecore_X_Atom *lst,
+                             unsigned int len)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    return ecore_x_window_prop_xid_get(win, atom, XA_ATOM, lst, len);
-} /* ecore_x_window_prop_atom_get */
+}
 
 /*
  * Get Atom (array) property
@@ -342,34 +365,39 @@ ecore_x_window_prop_atom_get(Ecore_X_Window win, Ecore_X_Atom atom,
  * Note: Return value 0 means that the property exists but has no elements.
  */
 EAPI int
-ecore_x_window_prop_atom_list_get(Ecore_X_Window win, Ecore_X_Atom atom,
+ecore_x_window_prop_atom_list_get(Ecore_X_Window win,
+                                  Ecore_X_Atom atom,
                                   Ecore_X_Atom **plst)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    return ecore_x_window_prop_xid_list_get(win, atom, XA_ATOM, plst);
-} /* ecore_x_window_prop_atom_list_get */
+}
 
 /*
  * Remove/add/toggle atom list item.
  */
 EAPI void
-ecore_x_window_prop_atom_list_change(Ecore_X_Window win, Ecore_X_Atom atom,
-                                     Ecore_X_Atom item, int op)
+ecore_x_window_prop_atom_list_change(Ecore_X_Window win,
+                                     Ecore_X_Atom atom,
+                                     Ecore_X_Atom item,
+                                     int op)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    ecore_x_window_prop_xid_list_change(win, atom, XA_ATOM, item, op);
-} /* ecore_x_window_prop_atom_list_change */
+}
 
 /*
  * Set Window (array) property
  */
 EAPI void
-ecore_x_window_prop_window_set(Ecore_X_Window win, Ecore_X_Atom atom,
-                               Ecore_X_Window *lst, unsigned int num)
+ecore_x_window_prop_window_set(Ecore_X_Window win,
+                               Ecore_X_Atom atom,
+                               Ecore_X_Window *lst,
+                               unsigned int num)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    ecore_x_window_prop_xid_set(win, atom, XA_WINDOW, lst, num);
-} /* ecore_x_window_prop_window_set */
+}
 
 /*
  * Get Window (array) property
@@ -380,12 +408,14 @@ ecore_x_window_prop_window_set(Ecore_X_Window win, Ecore_X_Atom atom,
  * Note: Return value 0 means that the property exists but has no elements.
  */
 EAPI int
-ecore_x_window_prop_window_get(Ecore_X_Window win, Ecore_X_Atom atom,
-                               Ecore_X_Window *lst, unsigned int len)
+ecore_x_window_prop_window_get(Ecore_X_Window win,
+                               Ecore_X_Atom atom,
+                               Ecore_X_Window *lst,
+                               unsigned int len)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    return ecore_x_window_prop_xid_get(win, atom, XA_WINDOW, lst, len);
-} /* ecore_x_window_prop_window_get */
+}
 
 /*
  * Get Window (array) property
@@ -396,50 +426,50 @@ ecore_x_window_prop_window_get(Ecore_X_Window win, Ecore_X_Atom atom,
  * Note: Return value 0 means that the property exists but has no elements.
  */
 EAPI int
-ecore_x_window_prop_window_list_get(Ecore_X_Window win, Ecore_X_Atom atom,
+ecore_x_window_prop_window_list_get(Ecore_X_Window win,
+                                    Ecore_X_Atom atom,
                                     Ecore_X_Window **plst)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    return ecore_x_window_prop_xid_list_get(win, atom, XA_WINDOW, plst);
-} /* ecore_x_window_prop_window_list_get */
+}
 
-/**
- * To be documented.
- *
- * FIXME: To be fixed.
- */
 EAPI Ecore_X_Atom
 ecore_x_window_prop_any_type(void)
 {
    return AnyPropertyType;
-} /* ecore_x_window_prop_any_type */
+}
 
 /**
- * To be documented.
- *
- * FIXME: To be fixed.
+ * @brief Set a property of Ecore_X_Window.
+ * @param win The window for which the property will be set.
+ * @param property The property of the window to be set.
+ * @param type The type of the property that will be set.
+ * @param size The size of the property that will be set.
+ * @param data The data of the property that will be set.
+ * @param number The size of data.
  */
 EAPI void
 ecore_x_window_prop_property_set(Ecore_X_Window win,
-                                 Ecore_X_Atom   property,
-                                 Ecore_X_Atom   type,
-                                 int            size,
-                                 void          *data,
-                                 int            number)
+                                 Ecore_X_Atom property,
+                                 Ecore_X_Atom type,
+                                 int size,
+                                 void *data,
+                                 int number)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (win == 0)
-      win = DefaultRootWindow(_ecore_x_disp);
+     win = DefaultRootWindow(_ecore_x_disp);
 
    if (size != 32)
-      XChangeProperty(_ecore_x_disp,
-                      win,
-                      property,
-                      type,
-                      size,
-                      PropModeReplace,
-                      (unsigned char *)data,
-                      number);
+     XChangeProperty(_ecore_x_disp,
+                     win,
+                     property,
+                     type,
+                     size,
+                     PropModeReplace,
+                     (unsigned char *)data,
+                     number);
    else
      {
         unsigned long *dat;
@@ -448,26 +478,34 @@ ecore_x_window_prop_property_set(Ecore_X_Window win,
         dat = malloc(sizeof(unsigned long) * number);
         if (dat)
           {
-             for (ptr = (int *)data, i = 0; i < number; i++) dat[i] = ptr[i];
+             for (ptr = (int *)data, i = 0; i < number; i++)
+               dat[i] = ptr[i];
              XChangeProperty(_ecore_x_disp, win, property, type, size,
                              PropModeReplace, (unsigned char *)dat, number);
              free(dat);
           }
      }
-} /* ecore_x_window_prop_property_set */
+}
 
 /**
- * To be documented.
- *
- * FIXME: To be fixed.
+ * @brief Get a property of Ecore_X_Window.
+ * @note If there aren't any data to be got the function return NULL.
+ *       If the function can't allocate the memory then 0 is returned.
+ * @param win The window for which the property will be got.
+ * @param property The property of the window that will be gotten.
+ * @param type The type of the property that will be gotten.
+ * @param size This parameter isn't in use.
+ * @param data The data of the property that will be gotten.
+ * @param num The size of property.
+ * @return size_ret The size of array that contains the property.
  */
 EAPI int
-ecore_x_window_prop_property_get(Ecore_X_Window  win,
-                                 Ecore_X_Atom    property,
-                                 Ecore_X_Atom    type,
-                                 int size        __UNUSED__,
+ecore_x_window_prop_property_get(Ecore_X_Window win,
+                                 Ecore_X_Atom property,
+                                 Ecore_X_Atom type,
+                                 int size __UNUSED__,
                                  unsigned char **data,
-                                 int            *num)
+                                 int *num)
 {
    Atom type_ret = 0;
    int ret, size_ret = 0;
@@ -476,23 +514,23 @@ ecore_x_window_prop_property_get(Ecore_X_Window  win,
 
    /* make sure these are initialized */
    if (num)
-      *num = 0;
+     *num = 0;
 
    if (data)
-      *data = NULL;
+     *data = NULL;
    else /* we can't store the retrieved data, so just return */
-      return 0;
+     return 0;
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (!win)
-      win = DefaultRootWindow(_ecore_x_disp);
+     win = DefaultRootWindow(_ecore_x_disp);
 
    ret = XGetWindowProperty(_ecore_x_disp, win, property, 0, LONG_MAX,
                             False, type, &type_ret, &size_ret,
                             &num_ret, &bytes, &prop_ret);
 
    if (ret != Success)
-      return 0;
+     return 0;
 
    if (!num_ret)
      {
@@ -508,38 +546,40 @@ ecore_x_window_prop_property_get(Ecore_X_Window  win,
 
    switch (size_ret) {
       case 8:
-         for (i = 0; i < num_ret; i++)
-            (*data)[i] = prop_ret[i];
-         break;
+        for (i = 0; i < num_ret; i++)
+          (*data)[i] = prop_ret[i];
+        break;
 
       case 16:
-         for (i = 0; i < num_ret; i++)
-            ((unsigned short *)*data)[i] = ((unsigned short *)prop_ret)[i];
-         break;
+        for (i = 0; i < num_ret; i++)
+          ((unsigned short *)*data)[i] = ((unsigned short *)prop_ret)[i];
+        break;
 
       case 32:
-         for (i = 0; i < num_ret; i++)
-            ((unsigned int *)*data)[i] = ((unsigned long *)prop_ret)[i];
-         break;
-     } /* switch */
+        for (i = 0; i < num_ret; i++)
+          ((unsigned int *)*data)[i] = ((unsigned long *)prop_ret)[i];
+        break;
+     }
 
    XFree(prop_ret);
 
    if (num)
-      *num = num_ret;
+     *num = num_ret;
 
    return size_ret;
-} /* ecore_x_window_prop_property_get */
+}
 
 EAPI void
-ecore_x_window_prop_property_del(Ecore_X_Window win, Ecore_X_Atom property)
+ecore_x_window_prop_property_del(Ecore_X_Window win,
+                                 Ecore_X_Atom property)
 {
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XDeleteProperty(_ecore_x_disp, win, property);
-} /* ecore_x_window_prop_property_del */
+}
 
 EAPI Ecore_X_Atom *
-ecore_x_window_prop_list(Ecore_X_Window win, int *num_ret)
+ecore_x_window_prop_list(Ecore_X_Window win,
+                         int *num_ret)
 {
    Ecore_X_Atom *atoms;
    Atom *atom_ret;
@@ -547,23 +587,24 @@ ecore_x_window_prop_list(Ecore_X_Window win, int *num_ret)
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (num_ret)
-      *num_ret = 0;
+     *num_ret = 0;
 
    atom_ret = XListProperties(_ecore_x_disp, win, &num);
    if (!atom_ret)
-      return NULL;
+     return NULL;
 
    atoms = malloc(num * sizeof(Ecore_X_Atom));
    if (atoms)
      {
-        for (i = 0; i < num; i++) atoms[i] = atom_ret[i];
+        for (i = 0; i < num; i++)
+          atoms[i] = atom_ret[i];
         if (num_ret)
-           *num_ret = num;
+          *num_ret = num;
      }
 
    XFree(atom_ret);
    return atoms;
-} /* ecore_x_window_prop_list */
+}
 
 /**
  * Set a window string property.
@@ -575,38 +616,38 @@ ecore_x_window_prop_list(Ecore_X_Window win, int *num_ret)
  */
 EAPI void
 ecore_x_window_prop_string_set(Ecore_X_Window win,
-                               Ecore_X_Atom   type,
-                               const char    *str)
+                               Ecore_X_Atom type,
+                               const char *str)
 {
    XTextProperty xtp;
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (win == 0)
-      win = DefaultRootWindow(_ecore_x_disp);
+     win = DefaultRootWindow(_ecore_x_disp);
 
    xtp.value = (unsigned char *)str;
    xtp.format = 8;
    xtp.encoding = ECORE_X_ATOM_UTF8_STRING;
    xtp.nitems = strlen(str);
    XSetTextProperty(_ecore_x_disp, win, &xtp, type);
-} /* ecore_x_window_prop_string_set */
+}
 
 /**
  * Get a window string property.
  * @param win The window
  * @param type The property
- *
- * Return window string property of a window. String must be free'd when done.
+ * @return Window string property of a window. String must be free'd when done.
  */
 EAPI char *
-ecore_x_window_prop_string_get(Ecore_X_Window win, Ecore_X_Atom type)
+ecore_x_window_prop_string_get(Ecore_X_Window win,
+                               Ecore_X_Atom type)
 {
    XTextProperty xtp;
    char *str = NULL;
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (win == 0)
-      win = DefaultRootWindow(_ecore_x_disp);
+     win = DefaultRootWindow(_ecore_x_disp);
 
    if (XGetTextProperty(_ecore_x_disp, win, &xtp, type))
      {
@@ -615,7 +656,7 @@ ecore_x_window_prop_string_get(Ecore_X_Window win, Ecore_X_Atom type)
         Status s;
 
         if (xtp.encoding == ECORE_X_ATOM_UTF8_STRING)
-           str = strdup((char *)xtp.value);
+          str = strdup((char *)xtp.value);
         else
           {
 #ifdef X_HAVE_UTF8_STRING
@@ -627,22 +668,22 @@ ecore_x_window_prop_string_get(Ecore_X_Window win, Ecore_X_Atom type)
 #endif /* ifdef X_HAVE_UTF8_STRING */
              if ((s == XLocaleNotSupported) ||
                  (s == XNoMemory) || (s == XConverterNotFound))
-                str = strdup((char *)xtp.value);
+               str = strdup((char *)xtp.value);
              else if ((s >= Success) && (items > 0))
-                str = strdup(list[0]);
+               str = strdup(list[0]);
 
              if (list)
-                XFreeStringList(list);
+               XFreeStringList(list);
           }
 
         XFree(xtp.value);
      }
 
    return str;
-} /* ecore_x_window_prop_string_get */
+}
 
 EAPI Eina_Bool
-ecore_x_window_prop_protocol_isset(Ecore_X_Window      win,
+ecore_x_window_prop_protocol_isset(Ecore_X_Window win,
                                    Ecore_X_WM_Protocol protocol)
 {
    Atom proto, *protos = NULL;
@@ -651,33 +692,37 @@ ecore_x_window_prop_protocol_isset(Ecore_X_Window      win,
 
    /* check for invalid values */
    if (protocol >= ECORE_X_WM_PROTOCOL_NUM)
-      return EINA_FALSE;
+     return EINA_FALSE;
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    proto = _ecore_x_atoms_wm_protocols[protocol];
 
    if (!XGetWMProtocols(_ecore_x_disp, win, &protos, &protos_count))
-      return ret;
+     return ret;
 
    for (i = 0; i < protos_count; i++)
-      if (protos[i] == proto)
-        {
-           ret = EINA_TRUE;
-           break;
-        }
+     if (protos[i] == proto)
+       {
+          ret = EINA_TRUE;
+          break;
+       }
 
    XFree(protos);
 
    return ret;
-} /* ecore_x_window_prop_protocol_isset */
+}
 
 /**
- * To be documented.
- *
- * FIXME: To be fixed.
+ * @brief Get a array containing the protocols of @a win
+ * @note If there aren't any properties to be counted or any protocols to get
+ *       then the function returns NULL.
+ * @param win The window for which protocol list will be got.
+ * @param num_ret Contains the number of elements of the array to be returned.
+ * @return The array that contains the protocols.
  */
 EAPI Ecore_X_WM_Protocol *
-ecore_x_window_prop_protocol_list_get(Ecore_X_Window win, int *num_ret)
+ecore_x_window_prop_protocol_list_get(Ecore_X_Window win,
+                                      int *num_ret)
 {
    Atom *protos = NULL;
    int i, protos_count = 0;
@@ -685,10 +730,10 @@ ecore_x_window_prop_protocol_list_get(Ecore_X_Window win, int *num_ret)
 
    LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (!XGetWMProtocols(_ecore_x_disp, win, &protos, &protos_count))
-      return NULL;
+     return NULL;
 
    if ((!protos) || (protos_count <= 0))
-      return NULL;
+     return NULL;
 
    prot_ret = calloc(1, protos_count * sizeof(Ecore_X_WM_Protocol));
    if (!prot_ret)
@@ -705,11 +750,11 @@ ecore_x_window_prop_protocol_list_get(Ecore_X_Window win, int *num_ret)
         for (j = 0; j < ECORE_X_WM_PROTOCOL_NUM; j++)
           {
              if (_ecore_x_atoms_wm_protocols[j] == protos[i])
-                prot_ret[i] = j;
+               prot_ret[i] = j;
           }
      }
    XFree(protos);
    *num_ret = protos_count;
    return prot_ret;
-} /* ecore_x_window_prop_protocol_list_get */
+}
 
